@@ -17,7 +17,7 @@
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 #ifdef HAVE_CVS_IDENT
-#ident "$Id: design_dump.cc,v 1.149 2004/10/04 01:10:52 steve Exp $"
+#ident "$Id: design_dump.cc,v 1.149.2.7 2006/08/23 04:09:14 steve Exp $"
 #endif
 
 # include "config.h"
@@ -171,7 +171,8 @@ void NetObj::dump_node_pins(ostream&o, unsigned ind) const
 
 	    if (pin(idx).is_linked()) {
 		  const Nexus*nex = pin(idx).nexus();
-		  o << nex << " " << nex->name();
+		  const char*nex_name = nex->name();
+		  o << nex << " " << (nex_name? nex_name : "????");
 	    }
 	    o << endl;
 
@@ -216,6 +217,23 @@ void NetCompare::dump_node(ostream&o, unsigned ind) const
       dump_obj_attr(o, ind+4);
 }
 
+void NetDecode::dump_node(ostream&o, unsigned ind) const
+{
+      o << setw(ind) << "" << "LPM_DECODE (NetDecode): " << name()
+	<< " ff=" << ff_->name() << ", word width=" << width() << endl;
+      dump_node_pins(o, ind+4);
+      dump_obj_attr(o, ind+4);
+}
+
+void NetDemux::dump_node(ostream&o, unsigned ind) const
+{
+      o << setw(ind) << "" << "LPM_DEMUX (NetDemux): " << name()
+	<< " bus width=" << width() << ", address width=" << awidth()
+	<< ", word count=" << size() << endl;
+      dump_node_pins(o, ind+4);
+      dump_obj_attr(o, ind+4);
+}
+
 void NetDivide::dump_node(ostream&o, unsigned ind) const
 {
       o << setw(ind) << "" << "NET_DIVIDE (NetDivide): " << name() << endl;
@@ -240,6 +258,7 @@ void NetMux::dump_node(ostream&o, unsigned ind) const
 {
       o << setw(ind) << "" << "Multiplexer (NetMux): " << name()
 	<< " scope=" << scope()->name() << endl;
+      o << setw(ind+6) << "" << "source=" << get_line() << endl;
       dump_node_pins(o, ind+4);
       dump_obj_attr(o, ind+4);
 }
@@ -272,7 +291,12 @@ void NetFF::dump_node(ostream&o, unsigned ind) const
 {
       o << setw(ind) << "" << "LPM_FF: " << name()
 	<< " scope=" << (scope()? scope()->name() : "")
-	<< " aset_value=" << aset_value_ << endl;
+	<< " aset_value=" << aset_value_
+	<< " sset_value=" << sset_value_;
+
+      if (demux_) o << " demux=" << demux_->name();
+
+      o << endl;
 
       dump_node_pins(o, ind+4);
       dump_obj_attr(o, ind+4);
@@ -355,8 +379,16 @@ void NetModulo::dump_node(ostream&o, unsigned ind) const
 
 void NetRamDq::dump_node(ostream&o, unsigned ind) const
 {
-      o << setw(ind) << "" << "LPM_RAM_DQ (" << mem_->name() << "): "
-	<< name() << endl;
+      o << setw(ind) << "" << "LPM_RAM_DQ (";
+
+      if (mem_)
+	    if (NetNet*tmp = mem_->reg_from_explode())
+		  o << "exploded mem=" << tmp->name();
+	    else
+		  o << "mem=" << mem_->name();
+
+      o << "): " << name() << endl;
+
       dump_node_pins(o, ind+4);
       dump_obj_attr(o, ind+4);
 }
@@ -537,6 +569,8 @@ void NetCase::dump(ostream&o, unsigned ind) const
 	    break;
       }
 
+      dump_proc_attr(o, ind+2);
+
       for (unsigned idx = 0 ;  idx < nitems_ ;  idx += 1) {
 	    o << setw(ind+2) << "";
 	    if (items_[idx].guard)
@@ -842,6 +876,15 @@ void NetProc::dump(ostream&o, unsigned ind) const
       o << setw(ind) << "" << "// " << typeid(*this).name() << endl;
 }
 
+void NetProc::dump_proc_attr(ostream&o, unsigned ind) const
+{
+      unsigned idx;
+      for (idx = 0 ;  idx < attr_cnt() ;  idx += 1) {
+	    o << setw(ind) << "" << "(* " << attr_key(idx) << " = "
+	      << attr_value(idx) << " *)" << endl;
+      }
+}
+
 /* Dump an expression that no one wrote a dump method for. */
 void NetExpr::dump(ostream&o) const
 {
@@ -1089,6 +1132,27 @@ void Design::dump(ostream&o) const
 
 /*
  * $Log: design_dump.cc,v $
+ * Revision 1.149.2.7  2006/08/23 04:09:14  steve
+ *  missing sig diagnostics.
+ *
+ * Revision 1.149.2.6  2006/07/10 00:21:50  steve
+ *  Add support for full_case attribute.
+ *
+ * Revision 1.149.2.5  2006/04/16 19:26:37  steve
+ *  Fix handling of exploded memories with partial or missing resets.
+ *
+ * Revision 1.149.2.4  2006/03/26 23:09:21  steve
+ *  Handle asynchronous demux/bit replacements.
+ *
+ * Revision 1.149.2.3  2006/03/12 07:34:16  steve
+ *  Fix the memsynth1 case.
+ *
+ * Revision 1.149.2.2  2006/02/19 00:11:31  steve
+ *  Handle synthesis of FF vectors with l-value decoder.
+ *
+ * Revision 1.149.2.1  2006/01/18 01:23:23  steve
+ *  Rework l-value handling to allow for more l-value type flexibility.
+ *
  * Revision 1.149  2004/10/04 01:10:52  steve
  *  Clean up spurious trailing white space.
  *
