@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2000-2004 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -16,8 +16,8 @@
  *    along with this program; if not, write to the Free Software
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
-#if !defined(WINNT) && !defined(macintosh)
-#ident "$Id: elab_sig.cc,v 1.20 2002/01/26 05:28:28 steve Exp $"
+#ifdef HAVE_CVS_IDENT
+#ident "$Id: elab_sig.cc,v 1.36 2004/09/27 22:34:10 steve Exp $"
 #endif
 
 # include "config.h"
@@ -29,6 +29,7 @@
 # include  "PGate.h"
 # include  "PTask.h"
 # include  "PWire.h"
+# include  "compiler.h"
 # include  "netlist.h"
 # include  "netmisc.h"
 # include  "util.h"
@@ -71,8 +72,8 @@ bool Module::elaborate_sig(Design*des, NetScope*scope) const
 
 	// Scan all the ports of the module, and make sure that each
 	// is connected to wires that have port declarations.
-      for (unsigned idx = 0 ;  idx < ports_.count() ;  idx += 1) {
-	    Module::port_t*pp = ports_[idx];
+      for (unsigned idx = 0 ;  idx < ports.count() ;  idx += 1) {
+	    Module::port_t*pp = ports[idx];
 	    if (pp == 0)
 		  continue;
 
@@ -122,7 +123,7 @@ bool Module::elaborate_sig(Design*des, NetScope*scope) const
 
 		  hname_t name = (*wt).first;
 
-		  if (! signal_is_in_port(ports_, name)) {
+		  if (! signal_is_in_port(ports, name)) {
 
 			cerr << cur->get_line() << ": error: Signal "
 			     << name << " has a declared direction "
@@ -174,7 +175,7 @@ bool Module::elaborate_sig(Design*des, NetScope*scope) const
       }
 
 
-      typedef map<string,PFunction*>::const_iterator mfunc_it_t;
+      typedef map<perm_string,PFunction*>::const_iterator mfunc_it_t;
 
       for (mfunc_it_t cur = funcs_.begin()
 		 ; cur != funcs_.end() ;  cur ++) {
@@ -195,7 +196,7 @@ bool Module::elaborate_sig(Design*des, NetScope*scope) const
 	// elaborate the ports of the tasks defined within this
 	// module. Run through them now.
 
-      typedef map<string,PTask*>::const_iterator mtask_it_t;
+      typedef map<perm_string,PTask*>::const_iterator mtask_it_t;
 
       for (mtask_it_t cur = tasks_.begin()
 		 ; cur != tasks_.end() ;  cur ++) {
@@ -210,31 +211,42 @@ bool Module::elaborate_sig(Design*des, NetScope*scope) const
 bool PGModule::elaborate_sig_mod_(Design*des, NetScope*scope,
 				  Module*rmod) const
 {
-	// Missing module instance names have already been rejected.
-      assert(get_name() != "");
+      bool flag = true;
+
+      NetScope::scope_vec_t instance = scope->instance_arrays[get_name()];
+
+      for (unsigned idx = 0 ;  idx < instance.count() ;  idx += 1) {
+	      // I know a priori that the elaborate_scope created the scope
+	      // already, so just look it up as a child of the current scope.
+	    NetScope*my_scope = instance[idx];
+	    assert(my_scope);
+
+	    if (my_scope->parent() != scope) {
+		  cerr << get_line() << ": internal error: "
+		       << "Instance " << my_scope->name()
+		       << " is in parent " << my_scope->parent()->name()
+		       << " instead of " << scope->name()
+		       << endl;
+	    }
+	    assert(my_scope->parent() == scope);
+
+	    if (! rmod->elaborate_sig(des, my_scope))
+		  flag = false;
 
-      if (msb_) {
-	    cerr << get_line() << ": sorry: Module instantiation arrays "
-		  "are not yet supported." << endl;
-	    des->errors += 1;
-	    return false;
       }
 
-
-	// I know a priori that the elaborate_scope created the scope
-	// already, so just look it up as a child of the current scope.
-      NetScope*my_scope = scope->child(get_name());
-      assert(my_scope);
-
-      return rmod->elaborate_sig(des, my_scope);
+      return flag;
 }
 
 /*
- * A function definition exists within an elaborated module.
+ * A function definition exists within an elaborated module. This
+ * matters when elaborating signals, as the ports of the function are
+ * created as signals/variables for each instance of the
+ * function. That is why PFunction has an elaborate_sig method.
  */
 void PFunction::elaborate_sig(Design*des, NetScope*scope) const
 {
-      string fname = scope->basename();
+      perm_string fname = scope->basename();
       assert(scope->type() == NetScope::FUNC);
 
 	/* Make sure the function has at least one input port. If it
@@ -248,21 +260,81 @@ void PFunction::elaborate_sig(Design*des, NetScope*scope) const
 	    des->errors += 1;
       }
 
-      svector<NetNet*>ports (ports_? ports_->count()+1 : 1);
+      NetNet*ret_sig = 0;
+      NetVariable*ret_real = 0;
 
-	/* Get the reg for the return value. I know the name of the
-	   reg variable, and I know that it is in this scope, so look
-	   it up directly. */
-      ports[0] = scope->find_signal(scope->basename());
-      if (ports[0] == 0) {
-	    cerr << get_line() << ": internal error: function scope "
-		 << scope->name() << " is missing return reg "
-		 << fname << "." << endl;
-	    scope->dump(cerr);
-	    des->errors += 1;
-	    return;
+	/* Create the signals/variables of the return value and write
+	   them into the function scope. */
+      switch (return_type_.type) {
+
+	  case PTF_REG:
+	    if (return_type_.range) {
+		  NetExpr*me = elab_and_eval(des, scope,
+					     (*return_type_.range)[0]);
+		  assert(me);
+		  NetExpr*le = elab_and_eval(des, scope,
+					     (*return_type_.range)[1]);
+		  assert(le);
+
+		  long mnum = 0, lnum = 0;
+		  if (NetEConst*tmp = dynamic_cast<NetEConst*>(me)) {
+			mnum = tmp->value().as_long();
+		  } else {
+			cerr << me->get_line() << ": error: "
+			      "Unable to evaluate constant expression "
+			     << *me << "." << endl;
+			des->errors += 1;
+		  }
+
+		  if (NetEConst*tmp = dynamic_cast<NetEConst*>(le)) {
+			lnum = tmp->value().as_long();
+		  } else {
+			cerr << le->get_line() << ": error: "
+			      "Unable to evaluate constant expression "
+			     << *le << "." << endl;
+			des->errors += 1;
+		  }
+
+		  ret_sig = new NetNet(scope, fname, NetNet::REG, mnum, lnum);
+
+	    } else {
+		  ret_sig = new NetNet(scope, fname, NetNet::REG);
+	    }
+	    ret_sig->set_line(*this);
+	    ret_sig->port_type(NetNet::POUTPUT);
+	    break;
+
+	  case PTF_INTEGER:
+	    ret_sig = new NetNet(scope, fname, NetNet::REG, INTEGER_WIDTH);
+	    ret_sig->set_line(*this);
+	    ret_sig->set_signed(true);
+	    ret_sig->set_isint(true);
+	    ret_sig->port_type(NetNet::POUTPUT);
+	    break;
+
+	  case PTF_TIME:
+	    ret_sig = new NetNet(scope, fname, NetNet::REG, 64);
+	    ret_sig->set_line(*this);
+	    ret_sig->set_signed(false);
+	    ret_sig->set_isint(false);
+	    ret_sig->port_type(NetNet::POUTPUT);
+	    break;
+
+	  case PTF_REAL:
+	  case PTF_REALTIME:
+	    ret_real = new NetVariable(fname);
+	    ret_real->set_line(*this);
+	    scope->add_variable(ret_real);
+	    break;
+
+	  default:
+	    cerr << get_line() << ": internal error: I don't know how "
+		 << "to deal with return type of function "
+		 << scope->basename() << "." << endl;
       }
 
+      svector<NetNet*>ports (ports_? ports_->count() : 0);
+
       if (ports_)
 	    for (unsigned idx = 0 ;  idx < ports_->count() ;  idx += 1) {
 
@@ -271,8 +343,8 @@ void PFunction::elaborate_sig(Design*des, NetScope*scope) const
 		       as two components: <func>.<port>. */
 
 		  hname_t path = (*ports_)[idx]->path();
-		  string pname = path.peek_name(1);
-		  string ppath = path.peek_name(0);
+		  perm_string pname = lex_strings.make(path.peek_name(1));
+		  perm_string ppath = lex_strings.make(path.peek_name(0));
 
 		  if (ppath != scope->basename()) {
 			cerr << get_line() << ": internal error: function "
@@ -288,14 +360,19 @@ void PFunction::elaborate_sig(Design*des, NetScope*scope) const
 			     << scope->name() << " is missing port "
 			     << pname << "." << endl;
 			scope->dump(cerr);
+			cerr << get_line() << ": Continuing..." << endl;
 			des->errors += 1;
 		  }
 
-		  ports[idx+1] = tmp;
+		  ports[idx] = tmp;
 	    }
 
 
-      NetFuncDef*def = new NetFuncDef(scope, ports);
+      NetFuncDef*def = 0;
+      if (ret_sig)  def = new NetFuncDef(scope, ret_sig, ports);
+      if (ret_real) def = new NetFuncDef(scope, ret_real, ports);
+
+      assert(def);
       scope->set_func_def(def);
 }
 
@@ -360,12 +437,13 @@ bool PGate::elaborate_sig(Design*des, NetScope*scope) const
 /*
  * Elaborate a source wire. The "wire" is the declaration of wires,
  * registers, ports and memories. The parser has already merged the
- * multiple properties of a wire (i.e. "input wire") so come the
+ * multiple properties of a wire (i.e., "input wire") so come the
  * elaboration this creates an object in the design that represent the
  * defined item.
  */
 void PWire::elaborate_sig(Design*des, NetScope*scope) const
 {
+
 	/* The parser may produce hierarchical names for wires. I here
 	   follow the scopes down to the base where I actually want to
 	   elaborate the NetNet object. */
@@ -373,7 +451,13 @@ void PWire::elaborate_sig(Design*des, NetScope*scope) const
         free(tmp_path.remove_tail_name());
 	for (unsigned idx = 0 ;  tmp_path.peek_name(idx) ;  idx += 1) {
 	      scope = scope->child(tmp_path.peek_name(idx));
-	      assert(scope);
+
+	      if (scope == 0) {
+		    cerr << get_line() << ": internal error: "
+			 << "Bad scope component for name "
+			 << hname_ << endl;
+		    assert(scope);
+	      }
 	}
       }
 
@@ -391,12 +475,12 @@ void PWire::elaborate_sig(Design*des, NetScope*scope) const
 	    svector<long>mnum (msb_.count());
 	    svector<long>lnum (msb_.count());
 	      /* There may be places where the signal is declared as a
-		 scaler. Count those here, for consistency check
+		 scalar. Count those here, for consistency check
 		 later. */
 	    unsigned count_scalars = 0;
 
 	      /* There may be multiple declarations of ranges, because
-		 the symbol may have its range declared in i.e. input
+		 the symbol may have its range declared in e.g., input
 		 and reg declarations. Calculate *all* the numbers
 		 here. I will resolve the values later. */
 
@@ -475,15 +559,21 @@ void PWire::elaborate_sig(Design*des, NetScope*scope) const
 
       }
 
+      unsigned nattrib = 0;
+      attrib_list_t*attrib_list = evaluate_attributes(attributes, nattrib,
+						      des, scope);
+
+	/* If the ident has idx expressions, then this is a
+	   memory. It can only have the idx registers after the msb
+	   and lsb expressions are filled. And, if it has one index,
+	   it has both. */
       if (lidx_ || ridx_) {
 	    assert(lidx_ && ridx_);
 
-	      // If the register has indices, then this is a
-	      // memory. Create the memory object.
-	    verinum*lval = lidx_->eval_const(des, scope);
-	    verinum*rval = ridx_->eval_const(des, scope);
+	    NetExpr*lexp = elab_and_eval(des, scope, lidx_);
+	    NetExpr*rexp = elab_and_eval(des, scope, ridx_);
 
-	    if ((lval == 0) || (rval == 0)) {
+	    if ((lexp == 0) || (rexp == 0)) {
 		  cerr << get_line() << ": internal error: There is "
 		       << "a problem evaluating indices for ``"
 		       << hname_.peek_tail_name() << "''." << endl;
@@ -491,34 +581,103 @@ void PWire::elaborate_sig(Design*des, NetScope*scope) const
 		  return;
 	    }
 
-	    assert(lval);
-	    assert(rval);
+	    NetEConst*lcon = dynamic_cast<NetEConst*> (lexp);
+	    NetEConst*rcon = dynamic_cast<NetEConst*> (rexp);
 
-	    string name = scope->name();
-	    name = name + "." + hname_.peek_tail_name();
+	    if ((lcon == 0) || (rcon == 0)) {
+		  cerr << get_line() << ": internal error: The indices "
+		       << "are not constant for memory ``"
+		       << hname_.peek_tail_name() << "''." << endl;
+		  des->errors += 1;
+		  return;
+	    }
 
-	    long lnum = lval->as_long();
-	    long rnum = rval->as_long();
-	    delete lval;
-	    delete rval;
-	    NetMemory*sig = new NetMemory(scope, name, wid, lnum, rnum);
+	    verinum lval = lcon->value();
+	    verinum rval = rcon->value();
+
+	    delete lexp;
+	    delete rexp;
+
+	    perm_string name = lex_strings.make(hname_.peek_tail_name());
+
+	    long lnum = lval.as_long();
+	    long rnum = rval.as_long();
+
+	    new NetMemory(scope, name, wid, lnum, rnum);
+	      // The constructor automatically adds the memory object
+	      // to the scope. Do I need to set line number information?
 
       } else {
 
-	      /* Make a hierarchical make for the signal. */
-	    string name = scope->name();
-	    name = name + "." + hname_.peek_tail_name();
-
+	    perm_string name = lex_strings.make(hname_.peek_tail_name());
 	    NetNet*sig = new NetNet(scope, name, wtype, msb, lsb);
 	    sig->set_line(*this);
 	    sig->port_type(port_type_);
 	    sig->set_signed(get_signed());
-	    sig->set_attributes(attributes);
+	    sig->set_isint(get_isint());
+
+	    for (unsigned idx = 0 ;  idx < nattrib ;  idx += 1)
+		  sig->attribute(attrib_list[idx].key, attrib_list[idx].val);
       }
 }
 
 /*
  * $Log: elab_sig.cc,v $
+ * Revision 1.36  2004/09/27 22:34:10  steve
+ *  Cleanup and factoring of autoconf.
+ *
+ * Revision 1.35  2004/09/05 17:44:41  steve
+ *  Add support for module instance arrays.
+ *
+ * Revision 1.34  2004/05/31 23:34:37  steve
+ *  Rewire/generalize parsing an elaboration of
+ *  function return values to allow for better
+ *  speed and more type support.
+ *
+ * Revision 1.33  2004/02/18 17:11:55  steve
+ *  Use perm_strings for named langiage items.
+ *
+ * Revision 1.32  2003/09/20 05:24:00  steve
+ *  Evaluate memory index constants using elab_and_eval.
+ *
+ * Revision 1.31  2003/07/15 03:49:22  steve
+ *  Spelling fixes.
+ *
+ * Revision 1.30  2003/06/24 01:38:02  steve
+ *  Various warnings fixed.
+ *
+ * Revision 1.29  2003/06/21 01:21:43  steve
+ *  Harmless fixup of warnings.
+ *
+ * Revision 1.28  2003/03/06 00:28:41  steve
+ *  All NetObj objects have lex_string base names.
+ *
+ * Revision 1.27  2003/01/30 16:23:07  steve
+ *  Spelling fixes.
+ *
+ * Revision 1.26  2003/01/27 05:09:17  steve
+ *  Spelling fixes.
+ *
+ * Revision 1.25  2002/08/12 01:34:59  steve
+ *  conditional ident string using autoconfig.
+ *
+ * Revision 1.24  2002/08/05 04:18:45  steve
+ *  Store only the base name of memories.
+ *
+ * Revision 1.23  2002/06/21 04:59:35  steve
+ *  Carry integerness throughout the compilation.
+ *
+ * Revision 1.22  2002/05/23 03:08:51  steve
+ *  Add language support for Verilog-2001 attribute
+ *  syntax. Hook this support into existing $attribute
+ *  handling, and add number and void value types.
+ *
+ *  Add to the ivl_target API new functions for access
+ *  of complex attributes attached to gates.
+ *
+ * Revision 1.21  2002/05/19 23:37:28  steve
+ *  Parse port_declaration_lists from the 2001 Standard.
+ *
  * Revision 1.20  2002/01/26 05:28:28  steve
  *  Detect scalar/vector declarion mismatch.
  *
@@ -551,46 +710,5 @@ void PWire::elaborate_sig(Design*des, NetScope*scope) const
  * Revision 1.11  2001/02/10 20:29:39  steve
  *  In the context of range declarations, use elab_and_eval instead
  *  of the less robust eval_const methods.
- *
- * Revision 1.10  2001/01/13 22:20:08  steve
- *  Parse parameters within nested scopes.
- *
- * Revision 1.9  2001/01/07 07:00:31  steve
- *  Detect port direction attached to non-ports.
- *
- * Revision 1.8  2001/01/04 04:47:51  steve
- *  Add support for << is signal indices.
- *
- * Revision 1.7  2000/12/11 00:31:43  steve
- *  Add support for signed reg variables,
- *  simulate in t-vvm signed comparisons.
- *
- * Revision 1.6  2000/12/04 17:37:04  steve
- *  Add Attrib class for holding NetObj attributes.
- *
- * Revision 1.5  2000/11/20 00:58:40  steve
- *  Add support for supply nets (PR#17)
- *
- * Revision 1.4  2000/09/07 22:37:48  steve
- *  ack, detect when lval fails.
- *
- * Revision 1.3  2000/07/30 18:25:43  steve
- *  Rearrange task and function elaboration so that the
- *  NetTaskDef and NetFuncDef functions are created during
- *  signal enaboration, and carry these objects in the
- *  NetScope class instead of the extra, useless map in
- *  the Design class.
- *
- * Revision 1.2  2000/07/14 06:12:57  steve
- *  Move inital value handling from NetNet to Nexus
- *  objects. This allows better propogation of inital
- *  values.
- *
- *  Clean up constant propagation  a bit to account
- *  for regs that are not really values.
- *
- * Revision 1.1  2000/05/02 16:27:38  steve
- *  Move signal elaboration to a seperate pass.
- *
  */
 
