@@ -17,7 +17,7 @@
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 #ifdef HAVE_CVS_IDENT
-#ident "$Id: elaborate.cc,v 1.308 2004/10/04 01:10:52 steve Exp $"
+#ident "$Id: elaborate.cc,v 1.308.2.6 2006/11/27 01:47:14 steve Exp $"
 #endif
 
 # include "config.h"
@@ -67,6 +67,21 @@ void PGate::elaborate(Design*des, NetScope*scope) const
 	    typeid(*this).name() << endl;
 }
 
+void process_attributes(Design*des, NetScope*scope,
+			Attrib*dest, const map<perm_string,PExpr*>&table)
+{
+      struct attrib_list_t*attrib_list = 0;
+      unsigned attrib_list_n = 0;
+      attrib_list = evaluate_attributes(table, attrib_list_n,
+					des, scope);
+
+      for (unsigned adx = 0 ;  adx < attrib_list_n ;  adx += 1)
+	    dest->attribute(attrib_list[adx].key,
+			    attrib_list[adx].val);
+
+      delete[]attrib_list;
+}
+
 /*
  * Elaborate the continuous assign. (This is *not* the procedural
  * assign.) Elaborate the lvalue and rvalue, and do the assignment.
@@ -1179,6 +1194,7 @@ NetProc* PAssign::elaborate(Design*des, NetScope*scope) const
 	    tmp->set_line(*this);
 
 	    NetESignal*sig = new NetESignal(tmp);
+	    sig->set_line(*this);
 
 	      /* Generate an assignment of the l-value to the temporary... */
 	    string n = scope->local_hsymbol();
@@ -1215,6 +1231,7 @@ NetProc* PAssign::elaborate(Design*des, NetScope*scope) const
 
 	      /* And build up the complex statement. */
 	    NetBlock*bl = new NetBlock(NetBlock::SEQU, 0);
+	    bl->set_line(*this);
 	    bl->append(a1);
 	    bl->append(st);
 
@@ -1318,6 +1335,7 @@ NetProc* PBlock::elaborate(Design*des, NetScope*scope) const
       }
 
       NetBlock*cur = new NetBlock(type, nscope);
+      cur->set_line(*this);
       bool fail_flag = false;
 
       if (nscope == 0)
@@ -1388,6 +1406,7 @@ NetProc* PCase::elaborate(Design*des, NetScope*scope) const
 
       NetCase*res = new NetCase(type_, expr, icount);
       res->set_line(*this);
+      process_attributes(des, scope, res, attributes);
 
 	/* Iterate over all the case items (guard/statement pairs)
 	   elaborating them. If the guard has no expression, then this
@@ -1403,8 +1422,12 @@ NetProc* PCase::elaborate(Design*des, NetScope*scope) const
 		    /* If there are no expressions, then this is the
 		       default case. */
 		  NetProc*st = 0;
-		  if (cur->stat)
+		  if (cur->stat) {
 			st = cur->stat->elaborate(des, scope);
+		  } else {
+			st = new NetBlock(NetBlock::SEQU, scope);
+			st->set_line(*this);
+		  }
 
 		  res->set_case(inum, 0, st);
 		  inum += 1;
@@ -1413,16 +1436,19 @@ NetProc* PCase::elaborate(Design*des, NetScope*scope) const
 
 		    /* If there are one or more expressions, then
 		       iterate over the guard expressions, elaborating
-		       a separate case for each. (Yes, the statement
-		       will be elaborated again for each.) */
+		       a separate case for each. If the statement is
+		       nul, then put an appropriate stub in place. */
 		  NetExpr*gu = 0;
 		  NetProc*st = 0;
 		  assert(cur->expr[e]);
 		  gu = elab_and_eval(des, scope, cur->expr[e]);
 
-		  if (cur->stat)
+		  if (cur->stat) {
 			st = cur->stat->elaborate(des, scope);
-
+		  } else {
+			st = new NetBlock(NetBlock::SEQU, scope);
+			st->set_line(*this);
+		  }
 		  res->set_case(inum, gu, st);
 		  inum += 1;
 	    }
@@ -1451,12 +1477,26 @@ NetProc* PCondit::elaborate(Design*des, NetScope*scope) const
       if (NetEConst*ce = dynamic_cast<NetEConst*>(expr)) {
 	    verinum val = ce->value();
 	    delete expr;
-	    if (val[0] == verinum::V1)
-		  return if_->elaborate(des, scope);
+
+	    verinum::V val_reduced = verinum::V0;
+	    for (unsigned idx = 0 ;  idx < val.len() ;  idx += 1)
+		  val_reduced = val_reduced | val[idx];
+
+	    if (val_reduced == verinum::V1)
+		  if (if_) {
+			return if_->elaborate(des, scope);
+		  } else {
+			NetBlock*tmp = new NetBlock(NetBlock::SEQU, 0);
+			tmp->set_line(*this);
+			return tmp;
+		  }
 	    else if (else_)
 		  return else_->elaborate(des, scope);
-	    else
-		  return new NetBlock(NetBlock::SEQU, 0);
+	    else {
+		  NetBlock*tmp = new NetBlock(NetBlock::SEQU, 0);
+		  tmp->set_line(*this);
+		  return tmp;
+	    }
       }
 
 	// If the condition expression is more then 1 bits, then
@@ -1625,6 +1665,7 @@ NetProc* PCallTask::elaborate_usr(Design*des, NetScope*scope) const
       }
 
       NetBlock*block = new NetBlock(NetBlock::SEQU, 0);
+      block->set_line(*this);
 
 
 	/* Detect the case where the definition of the task is known
@@ -1702,6 +1743,7 @@ NetProc* PCallTask::elaborate_usr(Design*des, NetScope*scope) const
 		  continue;
 
 	    NetESignal*sig = new NetESignal(port);
+	    sig->set_line(*this);
 
 	      /* Generate the assignment statement. */
 	    NetAssign*ass = new NetAssign(lv, sig);
@@ -2481,6 +2523,7 @@ void PTask::elaborate(Design*des, NetScope*task) const
       NetProc*st;
       if (statement_ == 0) {
 	    st = new NetBlock(NetBlock::SEQU, 0);
+	    st->set_line(*this);
 
       } else {
 
@@ -2769,6 +2812,24 @@ Design* elaborate(list<perm_string>roots)
 
 /*
  * $Log: elaborate.cc,v $
+ * Revision 1.308.2.6  2006/11/27 01:47:14  steve
+ *  Fix crash handling constant true conditional.
+ *
+ * Revision 1.308.2.5  2006/11/27 01:32:24  steve
+ *  Fix evaluate of constant condition expressions.
+ *
+ * Revision 1.308.2.4  2006/07/10 00:21:50  steve
+ *  Add support for full_case attribute.
+ *
+ * Revision 1.308.2.3  2006/06/12 00:16:51  steve
+ *  Add support for -Wunused warnings.
+ *
+ * Revision 1.308.2.2  2005/12/10 03:30:50  steve
+ *  Fix crash on block with assignments that assign lval to self.
+ *
+ * Revision 1.308.2.1  2005/11/13 22:28:14  steve
+ *  Do not panic if case statement is nul.
+ *
  * Revision 1.308  2004/10/04 01:10:52  steve
  *  Clean up spurious trailing white space.
  *
