@@ -17,24 +17,64 @@
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 #ifdef HAVE_CVS_IDENT
-#ident "$Id: expr_synth.cc,v 1.59 2004/06/30 02:16:26 steve Exp $"
+#ident "$Id: expr_synth.cc,v 1.59.2.11 2007/02/26 19:51:38 steve Exp $"
 #endif
 
 # include "config.h"
 
 # include  <iostream>
+# include  <typeinfo>
 
 # include  "netlist.h"
 # include  "netmisc.h"
+# include  "compiler.h"
 
 NetNet* NetExpr::synthesize(Design*des)
 {
       cerr << get_line() << ": internal error: cannot synthesize expression: "
 	   << *this << endl;
+      cerr << get_line() << ":               : typeid="
+	   << typeid(*this).name() << endl;
       des->errors += 1;
+
       return 0;
 }
 
+/*
+ * For the NetEBitSel expression, create a NetMux node that selects a
+ * bit from the input.
+ */
+NetNet* NetEBitSel::synthesize(Design*des)
+{
+      NetNet*net = sig_->synthesize(des);
+      assert(net);
+
+      NetNet*adr = idx_->synthesize(des);
+      if (adr == 0)
+	    return 0;
+
+      NetScope*scope = adr->scope();
+
+      NetNet*osig = new NetNet(scope, scope->local_symbol(),
+			       NetNet::WIRE, 1);
+      osig->set_line(*this);
+      osig->local_flag(true);
+
+      NetMux*mux = new NetMux(scope, scope->local_symbol(),
+			      1, net->pin_count(), adr->pin_count());
+      des->add_node(mux);
+
+      for (unsigned idx = 0 ;  idx < net->pin_count() ;  idx += 1)
+	    connect(mux->pin_Data(0, idx),  net->pin(idx));
+
+      for (unsigned idx = 0 ;  idx < adr->pin_count() ;  idx += 1)
+	    connect(mux->pin_Sel(idx), adr->pin(idx));
+
+      connect(mux->pin_Result(0), osig->pin(0));
+
+      return osig;
+}
+
 /*
  * Make an LPM_ADD_SUB device from addition operators.
  */
@@ -408,7 +448,7 @@ NetNet* NetEBDiv::synthesize(Design*des)
 
 	  default: {
 		cerr << get_line() << ": internal error: "
-		     << "NetEBDiv has unexpeced op() code: "
+		     << "NetEBDiv has unexpected op() code: "
 		     << op() << endl;
 		des->errors += 1;
 
@@ -472,13 +512,18 @@ NetNet* NetEBLogic::synthesize(Design*des)
 	    perm_string oname = scope->local_symbol();
 
 	    olog = new NetLogic(scope, oname, 3, NetLogic::AND);
+	    olog->set_line(*this);
 
 	    connect(osig->pin(0), olog->pin(0));
 	    des->add_node(olog);
 
-	      /* XXXX Here, I need to reduce the parameters with
-		 reduction or. */
+	      /* Here, I need to reduce the parameters with
+		 reduction or. Only do this if we must. */
+	    if (lsig->pin_count() > 1)
+		  lsig = reduction_or(des, lsig);
 
+	    if (rsig->pin_count() > 1)
+		  rsig = reduction_or(des, rsig);
 
 	      /* By this point, the left and right parameters have been
 		 reduced to single bit values. Now we just connect them to
@@ -486,6 +531,11 @@ NetNet* NetEBLogic::synthesize(Design*des)
 	    assert(lsig->pin_count() == 1);
 	    connect(lsig->pin(0), olog->pin(1));
 
+	    if (rsig->pin_count() != 1) {
+		  cerr << olog->get_line() << ": internal error: "
+		       << "right argument not reduced. expr=" << *this << endl;
+	    }
+
 	    assert(rsig->pin_count() == 1);
 	    connect(rsig->pin(0), olog->pin(2));
       }
@@ -566,7 +616,6 @@ NetNet* NetEBShift::synthesize(Design*des)
 			       NetNet::IMPLICIT, expr_width());
       osig->local_flag(true);
 
-      assert(op() == 'l');
       NetCLShift*dev = new NetCLShift(scope, scope->local_symbol(),
 				      osig->pin_count(),
 				      rsig->pin_count(),
@@ -640,6 +689,71 @@ NetNet* NetEConst::synthesize(Design*des)
       return osig;
 }
 
+NetNet* NetEMemory::synthesize(Design*des)
+{
+      NetScope*scope = mem_->scope();
+
+      NetNet*explode = mem_->explode_to_reg();
+      unsigned width = expr_width();
+
+      assert(idx_);
+      NetNet*addr = idx_->synthesize(des);
+
+      NetNet*osig = new NetNet(scope, scope->local_symbol(),
+			       NetNet::IMPLICIT,
+			       width);
+      osig->set_line(*this);
+
+      if (explode) {
+	    if (debug_synth)
+		  cerr << get_line() << ": debug: synthesize read of "
+		       << explode->pin_count() << " bit exploded memory." << endl;
+
+	      /* Only make a mux big enough to address the words that
+		 the address can generate. (If the address is
+		 0-extended, then only the low words are addressable.) */
+	    unsigned use_count = mem_->count();
+	    if (use_count > (1U << addr->pin_count())) {
+		  use_count = 1 << addr->pin_count();
+
+		  if (debug_synth)
+			cerr << get_line() << ": debug: "
+			     << "Index expression can only address "
+			     << use_count << " of "
+			     << mem_->count() << " words." << endl;
+	    }
+
+	      /* This is a reference to an exploded memory. So locate
+		 the reg vector and use the addr expression as a
+		 select into a MUX. */
+	    NetMux*mux = new NetMux(scope, scope->local_symbol(),
+				    width, use_count, addr->pin_count());
+	    des->add_node(mux);
+	    mux->set_line(*this);
+
+	    for (unsigned idx = 0 ;  idx < width ;  idx += 1)
+		  connect(mux->pin_Result(idx), osig->pin(idx));
+	    for (unsigned idx = 0 ;  idx < mux->sel_width() ;  idx += 1)
+		  connect(mux->pin_Sel(idx), addr->pin(idx));
+
+	    for (unsigned wrd = 0 ;  wrd < use_count ;  wrd += 1)
+		  for (unsigned idx = 0 ;  idx < width ;  idx += 1) {
+			unsigned bit = wrd*width + idx;
+			connect(mux->pin_Data(idx, wrd), explode->pin(bit));
+		  }
+
+	    if (debug_synth)
+		  cerr << get_line() << ": debug: synthesis done." << endl;
+
+      } else {
+	    cerr << get_line() << ": internal error: Synthesize memory "
+		 << "expression that is not exploded?" << endl;
+	    des->errors += 1;
+      }
+
+      return osig;
+}
+
 NetNet* NetECReal::synthesize(Design*des)
 {
       cerr << get_line() << ": error: Real constants are "
@@ -684,6 +798,96 @@ NetNet* NetEUBits::synthesize(Design*des)
       return osig;
 }
 
+NetNet* NetEUFunc::synthesize(Design*des)
+{
+      assert(func_);
+
+      NetFuncDef* def = func_->func_def();
+      assert(def);
+
+      assert(parms_.count() == def->port_count());
+
+      svector<NetNet*> inports (parms_.count());
+
+      unsigned errors = 0;
+      for (unsigned idx = 0 ;  idx < parms_.count() ;  idx += 1) {
+	    inports[idx] = parms_[idx]->synthesize(des);
+	    if (inports[idx] == 0)
+		  errors += 1;
+      }
+
+      if (errors > 0) {
+	    cerr << get_line() << ": error: "
+		 << "Cannot continue with function instance synthesis."
+		 << endl;
+	    return 0;
+      }
+
+      NetNet*out = def->synthesize(des, inports);
+
+      if (! out) {
+	    cerr << get_line() << ": error: "
+		 << "User defined functions do not synthesize." << endl;
+	    def->dump(cerr, 4);
+	    des->errors += 1;
+	    return 0;
+      }
+
+      return out;
+}
+
+NetNet* NetFuncDef::synthesize(Design*des, const svector<NetNet*>&inports_)
+{
+
+	/* First run synthesis for the function definition as if this
+	   where a toplevel process. This will create the logic that
+	   the function represents, but connected to the port nets for
+	   the function definition. We will detach those common ports
+	   later. */
+      NexusSet nex_set;
+      statement_->nex_output(nex_set);
+
+      const perm_string tmp1 = scope()->local_symbol();
+      NetNet*nex_out = new NetNet(scope(), tmp1, NetNet::WIRE,
+				  nex_set.count());
+      for (unsigned idx = 0 ;  idx < nex_out->pin_count() ;  idx += 1)
+	    connect(nex_set[idx], nex_out->pin(idx));
+
+      bool flag = statement_->synth_async_noaccum(des, scope(), false, 0,
+						  nex_out, nex_out);
+
+      if (!flag) {
+	    delete nex_out;
+	    return 0;
+      }
+
+	/* Connect the inports_ vectors to the input ports and detach
+	   the static arg ports themselves. This moves the input from
+	   the synthesized device from the static ports to the actual
+	   ports from the instance context. */
+      for (unsigned idx = 0 ;  idx < port_count() ;  idx += 1) {
+	    NetNet*in = inports_[idx];
+	    NetNet*arg = ports_[idx];
+
+	    assert(in->pin_count() == arg->pin_count());
+	    for (unsigned pin = 0 ;  pin < arg->pin_count() ;  pin += 1) {
+		  connect(in->pin(pin), arg->pin(pin));
+		  arg->pin(pin).unlink();
+	    }
+      }
+
+	/* Detach the output signal from the synthesized result. We
+	   use instead the nex_out that was returned from the
+	   synthesis of the function. This is how we account for the
+	   fact that the function may be synthesized multiple times to
+	   go into multiple expression. Each synthesis needs a unique
+	   output. */
+      for (unsigned idx = 0 ;  idx < result_sig_->pin_count() ;  idx += 1)
+	    result_sig_->pin(idx).unlink();
+
+      return nex_out;
+}
+
 NetNet* NetEUReduce::synthesize(Design*des)
 {
       NetNet*isig = expr_->synthesize(des);
@@ -811,6 +1015,7 @@ NetNet* NetETernary::synthesize(Design *des)
 
       perm_string oname = csig->scope()->local_symbol();
       NetMux *mux = new NetMux(csig->scope(), oname, width, 2, 1);
+      mux->set_line(*this);
       for (unsigned idx = 0 ;  idx < width;  idx += 1) {
 	    connect(tsig->pin(idx), mux->pin_Data(idx, 1));
 	    connect(fsig->pin(idx), mux->pin_Data(idx, 0));
@@ -854,6 +1059,21 @@ NetNet* NetETernary::synthesize(Design *des)
  */
 NetNet* NetESignal::synthesize(Design*des)
 {
+      if (warn_unused) {
+	    if (net_->peek_lref() == 0 && net_->type()==NetNet::REG) {
+		  cerr << get_line() << ": warning: "
+		       << "reg " << net_->name()
+		       << " is used by logic but never assigned." << endl;
+	    }
+
+	    if (net_->peek_lref() == 0 && net_->type()==NetNet::INTEGER) {
+		  cerr << get_line() << ": warning: "
+		       << "reg " << net_->name()
+		       << " is used by logic but never assigned." << endl;
+	    }
+      }
+
+	/* If there is no part select, then the synthesis is trivial. */
       if ((lsi_ == 0) && (msi_ == (net_->pin_count() - 1)))
 	    return net_;
 
@@ -866,6 +1086,7 @@ NetNet* NetESignal::synthesize(Design*des)
       perm_string name = scope->local_symbol();
       NetNet*tmp = new NetNet(scope, name, NetNet::WIRE, wid);
       tmp->local_flag(true);
+      tmp->set_line(*this);
 
       for (unsigned idx = 0 ;  idx < wid ;  idx += 1)
 	    connect(tmp->pin(idx), net_->pin(idx+lsi_));
@@ -875,6 +1096,39 @@ NetNet* NetESignal::synthesize(Design*des)
 
 /*
  * $Log: expr_synth.cc,v $
+ * Revision 1.59.2.11  2007/02/26 19:51:38  steve
+ *  Spelling fixes (larry doolittle)
+ *
+ * Revision 1.59.2.10  2006/11/26 01:54:05  steve
+ *  Add synthesis of user defined functions.
+ *
+ * Revision 1.59.2.9  2006/08/08 02:17:48  steve
+ *  Improved nexus management performance.
+ *
+ * Revision 1.59.2.8  2006/06/15 01:57:26  steve
+ *  Handle simple memory addressing in expression synthesis.
+ *
+ * Revision 1.59.2.7  2006/06/14 03:02:54  steve
+ *  synthesis for NetEBitSel.
+ *
+ * Revision 1.59.2.6  2006/06/12 00:16:52  steve
+ *  Add support for -Wunused warnings.
+ *
+ * Revision 1.59.2.5  2006/05/15 03:55:22  steve
+ *  Fix synthesis of expressions with land of vectors.
+ *
+ * Revision 1.59.2.4  2006/04/10 03:43:39  steve
+ *  Exploded memories accessed by constant indices.
+ *
+ * Revision 1.59.2.3  2006/03/16 05:39:20  steve
+ *  Right shifts really are allowed.
+ *
+ * Revision 1.59.2.2  2005/09/11 02:56:37  steve
+ *  Attach line numbers to NetMux devices.
+ *
+ * Revision 1.59.2.1  2005/02/19 16:39:31  steve
+ *  Spellig fixes.
+ *
  * Revision 1.59  2004/06/30 02:16:26  steve
  *  Implement signed divide and signed right shift in nets.
  *
@@ -883,82 +1137,5 @@ NetNet* NetESignal::synthesize(Design*des)
  *
  * Revision 1.57  2004/06/12 15:00:02  steve
  *  Support / and % in synthesized contexts.
- *
- * Revision 1.56  2004/06/01 01:04:57  steve
- *  Fix synthesis method for logical and/or
- *
- * Revision 1.55  2004/02/20 18:53:35  steve
- *  Addtrbute keys are perm_strings.
- *
- * Revision 1.54  2004/02/18 17:11:56  steve
- *  Use perm_strings for named langiage items.
- *
- * Revision 1.53  2004/02/15 04:23:48  steve
- *  Fix evaluation of compare to constant expression.
- *
- * Revision 1.52  2003/11/10 19:39:20  steve
- *  Remove redundant scope tokens.
- *
- * Revision 1.51  2003/10/27 06:04:21  steve
- *  More flexible width handling for synthesized add.
- *
- * Revision 1.50  2003/09/26 02:44:27  steve
- *  Assure ternary arguments are wide enough.
- *
- * Revision 1.49  2003/09/03 23:31:36  steve
- *  Support synthesis of constant downshifts.
- *
- * Revision 1.48  2003/08/28 04:11:18  steve
- *  Spelling patch.
- *
- * Revision 1.47  2003/08/09 03:23:40  steve
- *  Add support for IVL_LPM_MULT device.
- *
- * Revision 1.46  2003/07/26 03:34:42  steve
- *  Start handling pad of expressions in code generators.
- *
- * Revision 1.45  2003/06/24 01:38:02  steve
- *  Various warnings fixed.
- *
- * Revision 1.44  2003/04/19 04:52:56  steve
- *  Less picky about expression widths while synthesizing ternary.
- *
- * Revision 1.43  2003/04/08 05:07:15  steve
- *  Detect constant shift distances in synthesis.
- *
- * Revision 1.42  2003/04/08 04:33:55  steve
- *  Synthesize shift expressions.
- *
- * Revision 1.41  2003/03/06 00:28:41  steve
- *  All NetObj objects have lex_string base names.
- *
- * Revision 1.40  2003/02/26 01:29:24  steve
- *  LPM objects store only their base names.
- *
- * Revision 1.39  2003/01/30 16:23:07  steve
- *  Spelling fixes.
- *
- * Revision 1.38  2003/01/26 21:15:58  steve
- *  Rework expression parsing and elaboration to
- *  accommodate real/realtime values and expressions.
- *
- * Revision 1.37  2002/11/17 23:37:55  steve
- *  Magnitude compare to 0.
- *
- * Revision 1.36  2002/08/12 01:34:59  steve
- *  conditional ident string using autoconfig.
- *
- * Revision 1.35  2002/07/07 22:31:39  steve
- *  Smart synthesis of binary AND expressions.
- *
- * Revision 1.34  2002/07/05 21:26:17  steve
- *  Avoid emitting to vvp local net symbols.
- *
- * Revision 1.33  2002/05/26 01:39:02  steve
- *  Carry Verilog 2001 attributes with processes,
- *  all the way through to the ivl_target API.
- *
- *  Divide signal reference counts between rval
- *  and lval references.
  */
 
