@@ -19,7 +19,7 @@
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 #ifdef HAVE_CVS_IDENT
-#ident "$Id: ivl_target.h,v 1.126 2004/10/04 01:10:53 steve Exp $"
+#ident "$Id: ivl_target.h,v 1.126.2.5 2006/04/16 19:26:37 steve Exp $"
 #endif
 
 #ifdef __cplusplus
@@ -226,8 +226,11 @@ typedef enum ivl_lpm_type_e {
       IVL_LPM_CMP_GE =  1,
       IVL_LPM_CMP_GT =  2,
       IVL_LPM_CMP_NE = 11,
+      IVL_LPM_DECODE = 15,
+      IVL_LPM_DEMUX  = 16,
       IVL_LPM_DIVIDE = 12,
       IVL_LPM_FF     =  3,
+      IVL_LPM_LATCH  = 17,
       IVL_LPM_MOD    = 13,
       IVL_LPM_MULT   =  4,
       IVL_LPM_MUX    =  5,
@@ -629,6 +632,11 @@ extern const char* ivl_udp_name(ivl_udp_t net);
  *    the LPM type, but it generally has to do with the width of the
  *    output data path.
  *
+ * ivl_lpm_attr_cnt
+ * ivl_lpm_attr_val
+ *    These functions access attributes attached to LPM devices. These
+ *    attributes may be explicit attributes collected from the Verilog
+ *    source, or implicit attributes generated by the compiler.
  *
  * These functions apply to a subset of the LPM devices, or may have
  * varying meaning depending on the device:
@@ -642,13 +650,18 @@ extern const char* ivl_udp_name(ivl_udp_t net);
  *    Return the input data nexus for device types that have a second
  *    input vector. For example, arithmetic devices are like this.
  *
+ * ivl_lpm_decode
+ *    Return the DECODER associated with this LPM_FF device. The
+ *    decoder for a FF takes an input address and generates an enable
+ *    input for no more then 1 bit (can be none) of the FF device.
+ *
  * ivl_lpm_q
  *    Return the output data nexus for device types that have a single
  *    output vector. This is most devices, it turns out.
  *
  * ivl_lpm_selects
- *    This is the size of the select input for a LPM_MUX device, or the
- *    address bus width of an LPM_RAM.
+ *    This is the size of the select input for a LPM_MUX or LPM_DECODE
+ *    device, or the address bus width of an LPM_RAM.
  *
  * ivl_lpm_signed
  *    Arithmetic LPM devices may be signed or unsigned if there is a
@@ -658,6 +671,64 @@ extern const char* ivl_udp_name(ivl_udp_t net);
  *    In addition to a width, some devices have a size. The size is
  *    often the number of inputs per out, i.e., the number of inputs
  *    per bit for a MUX.
+ *
+ * SEMANTIC NOTES:
+ *
+ * - IVL_LPM_FF
+ * The IVL_LPM_FF and IVL_LPM_DECODE devices are closely related. If
+ * the ivl_lpm_decode function returns a non-nil value, then the
+ * decoder represents an extra ENABLE-like input, where exactly <N>
+ * bits of the width of the FF is enabled. The ivl_lpm_width of the
+ * decoder defines <N>.
+ *
+ * The decoder inputs are the address that selects the FF to be
+ * enabled, and the ivl_lpm_selects for the decoder gives the width of
+ * the address. The address of the LSB of the memory, then, is the
+ * word width times the input address. For a simple l-value bit
+ * select, the word width <N> will be 1, and the address goes directly
+ * to the bit. Otherwise, <N>*address gets to the first bit of the word.
+ *
+ * The core compiler generates these attributes in certain cases:
+ *
+ *  - ivl:clock_polarity
+ *    If present, the string value can be "INVERT". That indicates
+ *    that the clock is negedge sensitive instead of the default
+ *    posedge sensitive.
+ *
+ * - IVL_LPM_RAM
+ * The IVL_LPM_RAM may also appear as a READ port for a FF array. In
+ * this case, the IVL_LPM_RAM device has an ivl_lpm_width that is the
+ * width of the word, and an ivl_lpm_size that is the number of words
+ * in the array. In effect, the IVL_LPM_RAM reorganizes a vector into
+ * an array of words. If this is happening, then the ivl_lpm_memory
+ * will return 0.
+ *
+ * The ivl_lpm_q function gets the output nexa of the read port. The
+ * "q" port has as many bits as the width. The ivl_lpm_data2 function
+ * gets the nexa of the input, with the sdx the word address and the
+ * idx the bit within the word.
+ *
+ * - IVL_LPM_DEMUX
+ * This device is a form of bit replacement. The device has a data bus
+ * width (ivl_lpm_width) and an address width. The Data outputs are
+ * ivl_lpm_q and have the data bus width. The ivl_lpm_data functions are
+ * inputs to the device. Normally, the device passes the data inputs
+ * through to the Q output.
+ *
+ * The IVL_LPM_DEMUX also has a size (ivl_lpm_size) which is the
+ * number of addressable units in the device. If the device is bit-
+ * addressable, then the size is the same as the width. I.e. if
+ * ivl_lpm_width == ivl_lpm_size, then the address selects a single
+ * bit. If ivl_lpm_width/ivl_lpm_size == 8, then the address selects
+ * words of 8 bits and there are ivl_lpm_size of them. The
+ * ivl_lpm_width will always be an exact multiple of
+ * ivl_lpm_size. This is slightly different from the meaning of width
+ * in memory ports, but reflects that the DEMUX manipulates fixed
+ * width ranges within a single vector.
+ *
+ * The ivl_lpm_select inputs address a bit of the device. The
+ * addressed bit is substituted by the replacement bit. This
+ * replacement bit comes from the ivl_lpm_datab(net,0) nexus.
  */
 
 extern const char*    ivl_lpm_name(ivl_lpm_t net); /* (Obsolete) */
@@ -667,6 +738,9 @@ extern int            ivl_lpm_signed(ivl_lpm_t net);
 extern ivl_lpm_type_t ivl_lpm_type(ivl_lpm_t net);
 extern unsigned       ivl_lpm_width(ivl_lpm_t net);
 
+extern unsigned        ivl_lpm_attr_cnt(ivl_lpm_t net);
+extern ivl_attribute_t ivl_lpm_attr_val(ivl_lpm_t net, unsigned idx);
+
   /* IVL_LPM_FF */
 extern ivl_nexus_t ivl_lpm_async_clr(ivl_lpm_t net);
 extern ivl_nexus_t ivl_lpm_async_set(ivl_lpm_t net);
@@ -676,26 +750,29 @@ extern ivl_nexus_t ivl_lpm_sync_set(ivl_lpm_t net);
 extern ivl_expr_t  ivl_lpm_sset_value(ivl_lpm_t net);
   /* IVL_LPM_FF IVL_LPM_RAM */
 extern ivl_nexus_t ivl_lpm_clk(ivl_lpm_t net);
+  /* IVL_LPM_FF */
+extern ivl_lpm_t   ivl_lpm_decode(ivl_lpm_t net);
   /* IVL_LPM_UFUNC */
 extern ivl_scope_t  ivl_lpm_define(ivl_lpm_t net);
   /* IVL_LPM_FF IVL_LPM_RAM */
 extern ivl_nexus_t ivl_lpm_enable(ivl_lpm_t net);
-  /* IVL_LPM_ADD IVL_LPM_FF IVL_LPM_MULT IVL_LPM_RAM IVL_LPM_SUB */
+  /* IVL_LPM_ADD IVL_LPM_DEMUX IVL_LPM_FF IVL_LPM_MULT */
+  /*  IVL_LPM_RAM IVL_LPM_SUB */
 extern ivl_nexus_t ivl_lpm_data(ivl_lpm_t net, unsigned idx);
-  /* IVL_LPM_ADD IVL_LPM_MULT IVL_LPM_SUB */
+  /* IVL_LPM_ADD IVL_LPM_DEMUX IVL_LPM_MULT IVL_LPM_SUB */
   /* IVL_LPM_MUX IVL_LPM_UFUNC */
 extern ivl_nexus_t ivl_lpm_datab(ivl_lpm_t net, unsigned idx);
 extern ivl_nexus_t ivl_lpm_data2(ivl_lpm_t net, unsigned sdx, unsigned idx);
   /* IVL_LPM_UFUNC */
 extern unsigned ivl_lpm_data2_width(ivl_lpm_t net, unsigned sdx);
-  /* IVL_LPM_ADD IVL_LPM_FF IVL_LPM_MULT IVL_LPM_RAM IVL_LPM_SUB
+  /* IVL_LPM_ADD IVL_LPM_DEMUX IVL_LPM_FF IVL_LPM_MULT IVL_LPM_RAM IVL_LPM_SUB
      IVL_LPM_UFUNC */
 extern ivl_nexus_t ivl_lpm_q(ivl_lpm_t net, unsigned idx);
-  /* IVL_LPM_MUX IVL_LPM_RAM */
+  /* IVL_LPM_MUX IVL_LPM_DECODE IVL_LPM_DEMUX IVL_LPM_RAM */
 extern unsigned ivl_lpm_selects(ivl_lpm_t net);
-  /* IVL_LPM_MUX IVL_LPM_RAM */
+  /* IVL_LPM_MUX IVL_LPM_DECODE IVL_LPM_DEMUX IVL_LPM_RAM */
 extern ivl_nexus_t ivl_lpm_select(ivl_lpm_t net, unsigned idx);
-  /* IVL_LPM_MUX */
+  /* IVL_LPM_DEMUX, IVL_LPM_MUX, IVL_LPM_RAM */
 extern unsigned ivl_lpm_size(ivl_lpm_t net);
   /* IVL_LPM_RAM */
 extern ivl_memory_t ivl_lpm_memory(ivl_lpm_t net);
@@ -1236,6 +1313,21 @@ _END_DECL
 
 /*
  * $Log: ivl_target.h,v $
+ * Revision 1.126.2.5  2006/04/16 19:26:37  steve
+ *  Fix handling of exploded memories with partial or missing resets.
+ *
+ * Revision 1.126.2.4  2006/03/26 23:09:22  steve
+ *  Handle asynchronous demux/bit replacements.
+ *
+ * Revision 1.126.2.3  2006/03/12 07:34:16  steve
+ *  Fix the memsynth1 case.
+ *
+ * Revision 1.126.2.2  2006/02/25 05:03:28  steve
+ *  Add support for negedge FFs by using attributes.
+ *
+ * Revision 1.126.2.1  2006/02/19 00:11:31  steve
+ *  Handle synthesis of FF vectors with l-value decoder.
+ *
  * Revision 1.126  2004/10/04 01:10:53  steve
  *  Clean up spurious trailing white space.
  *
