@@ -19,7 +19,7 @@ const char COPYRIGHT[] =
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 #ifdef HAVE_CVS_IDENT
-#ident "$Id: main.cc,v 1.86 2004/10/04 01:10:53 steve Exp $"
+#ident "$Id: main.cc,v 1.86.2.3 2006/06/12 00:16:53 steve Exp $"
 #endif
 
 # include "config.h"
@@ -75,7 +75,7 @@ extern "C" const char*optarg;
 /* Count errors detected in flag processing. */
 unsigned flag_errors = 0;
 
-const char VERSION[] = "$Name: v0_8 $";
+const char VERSION[] = "$Name:  $";
 
 const char*basedir = ".";
 
@@ -103,6 +103,7 @@ FILE *depend_file = NULL;
 bool warn_implicit  = false;
 bool warn_timescale = false;
 bool warn_portbinding = false;
+bool warn_unused = false;
 
 bool error_implicit = false;
 
@@ -112,6 +113,9 @@ bool error_implicit = false;
 bool debug_scopes = false;
 bool debug_eval_tree = false;
 bool debug_elaborate = false;
+bool debug_synth = false;
+bool debug_cprop = false;
+
 /*
  * Verbose messages enabled.
  */
@@ -314,6 +318,12 @@ static void read_iconfig_file(const char*ipath)
 		  } else if (strcmp(cp,"elaborate") == 0) {
 			debug_elaborate = true;
 			cerr << "debug: Enable elaborate debug" << endl;
+		  } else if (strcmp(cp,"synth") == 0) {
+			debug_synth = true;
+			cerr << "debug: Enable synthesis debug" << endl;
+		  } else if (strcmp(cp,"cprop") == 0) {
+			debug_cprop = true;
+			cerr << "debug: Enable cprop debug" << endl;
 		  } else {
 		  }
 
@@ -376,6 +386,9 @@ static void read_iconfig_file(const char*ipath)
 		      case 't':
 			warn_timescale = true;
 			break;
+		      case 'u':
+			warn_unused = true;
+			break;
 		      default:
 			break;
 		  }
@@ -744,6 +757,15 @@ int main(int argc, char*argv[])
 
 /*
  * $Log: main.cc,v $
+ * Revision 1.86.2.3  2006/06/12 00:16:53  steve
+ *  Add support for -Wunused warnings.
+ *
+ * Revision 1.86.2.2  2006/04/23 04:25:45  steve
+ *  Add cprop debugging.
+ *
+ * Revision 1.86.2.1  2006/04/01 01:37:24  steve
+ *  Add synth debug flag
+ *
  * Revision 1.86  2004/10/04 01:10:53  steve
  *  Clean up spurious trailing white space.
  *
