@@ -17,7 +17,7 @@
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 #ifdef HAVE_CVS_IDENT
-#ident "$Id: t-dll-api.cc,v 1.108 2004/10/04 01:10:55 steve Exp $"
+#ident "$Id: t-dll-api.cc,v 1.108.2.5 2006/04/16 19:26:40 steve Exp $"
 #endif
 
 # include "config.h"
@@ -620,6 +620,19 @@ extern "C" const char* ivl_udp_name(ivl_udp_t net)
       return net->name;
 }
 
+extern "C" unsigned ivl_lpm_attr_cnt(ivl_lpm_t net)
+{
+      return net->nattr;
+}
+
+extern "C" ivl_attribute_t ivl_lpm_attr_val(ivl_lpm_t net, unsigned idx)
+{
+      if (idx >= net->nattr)
+	    return 0;
+      else
+	    return net->attr + idx;
+}
+
 extern "C" const char* ivl_lpm_basename(ivl_lpm_t net)
 {
       return net->name;
@@ -752,6 +765,10 @@ extern "C" ivl_nexus_t ivl_lpm_data(ivl_lpm_t net, unsigned idx)
 	    assert(idx < net->u_.arith.width);
 	    return net->u_.arith.a[idx];
 
+	  case IVL_LPM_DEMUX:
+	    assert(idx < net->u_.demux.width);
+	    return net->u_.demux.d[idx];
+
 	  case IVL_LPM_SHIFTL:
 	  case IVL_LPM_SHIFTR:
 	    assert(idx < net->u_.shift.width);
@@ -788,6 +805,10 @@ extern "C" ivl_nexus_t ivl_lpm_datab(ivl_lpm_t net, unsigned idx)
 	    assert(idx < net->u_.arith.width);
 	    return net->u_.arith.b[idx];
 
+	  case IVL_LPM_DEMUX:
+	    assert(idx < net->u_.demux.width/net->u_.demux.size);
+	    return net->u_.demux.bit_in[idx];
+
 	  default:
 	    assert(0);
 	    return 0;
@@ -813,6 +834,22 @@ extern "C" ivl_nexus_t ivl_lpm_data2(ivl_lpm_t net, unsigned sdx, unsigned idx)
 		return net->u_.ufunc.pins[base+idx];
 	  }
 
+	  case IVL_LPM_RAM:
+	    if (net->u_.ff.a.mem == 0) {
+		    // This is an exploded RAM, so we use sdx and idx
+		    // to address a nexa into the exploded ram.
+		  if (sdx >= net->u_.ff.scnt)
+			return 0;
+		  if (idx >= net->u_.ff.width)
+			return 0;
+		  unsigned adr = sdx * net->u_.ff.width + idx;
+		  return net->u_.ff.d.pins[adr];
+
+	    } else {
+		    // Normal RAM port does not have data2 nexa
+		  return 0;
+	    }
+
 	  default:
 	    assert(0);
 	    return 0;
@@ -833,6 +870,18 @@ extern "C" unsigned ivl_lpm_data2_width(ivl_lpm_t net, unsigned sdx)
       }
 }
 
+extern "C" ivl_lpm_t ivl_lpm_decode(ivl_lpm_t net)
+{
+      assert(net);
+      switch (net->type) {
+	  case IVL_LPM_FF:
+	    return net->u_.ff.a.decode;
+	  default:
+	    assert(0);
+	    return 0;
+      }
+}
+
 /*
  * This function returns the hierarchical name for the LPM device. The
  * name needs to be built up from the scope name and the lpm base
@@ -898,6 +947,10 @@ extern "C" ivl_nexus_t ivl_lpm_q(ivl_lpm_t net, unsigned idx)
 	    else
 		  return net->u_.mux.q.pins[idx];
 
+	  case IVL_LPM_DEMUX:
+	    assert(idx < net->u_.demux.width);
+	    return net->u_.demux.q[idx];
+
 	  case IVL_LPM_SHIFTL:
 	  case IVL_LPM_SHIFTR:
 	    assert(idx < net->u_.shift.width);
@@ -929,6 +982,7 @@ extern "C" ivl_nexus_t ivl_lpm_select(ivl_lpm_t net, unsigned idx)
 	    else
 		  return net->u_.ff.s.pins[idx];
 
+	  case IVL_LPM_DECODE:
 	  case IVL_LPM_MUX:
 	    assert(idx < net->u_.mux.swid);
 	    if (net->u_.mux.swid == 1)
@@ -936,6 +990,10 @@ extern "C" ivl_nexus_t ivl_lpm_select(ivl_lpm_t net, unsigned idx)
 	    else
 		  return net->u_.mux.s.pins[idx];
 
+	  case IVL_LPM_DEMUX:
+	    assert(idx < net->u_.demux.awid);
+	    return net->u_.demux.a[idx];
+
 	  case IVL_LPM_SHIFTL:
 	  case IVL_LPM_SHIFTR:
 	    assert(idx < net->u_.shift.select);
@@ -952,8 +1010,11 @@ extern "C" unsigned ivl_lpm_selects(ivl_lpm_t net)
       switch (net->type) {
 	  case IVL_LPM_RAM:
 	    return net->u_.ff.swid;
+	  case IVL_LPM_DECODE:
 	  case IVL_LPM_MUX:
 	    return net->u_.mux.swid;
+	  case IVL_LPM_DEMUX:
+	    return net->u_.demux.awid;
 	  case IVL_LPM_SHIFTL:
 	  case IVL_LPM_SHIFTR:
 	    return net->u_.shift.select;
@@ -970,6 +1031,7 @@ extern "C" int ivl_lpm_signed(ivl_lpm_t net)
 	  case IVL_LPM_FF:
 	  case IVL_LPM_RAM:
 	  case IVL_LPM_MUX:
+	  case IVL_LPM_DEMUX:
 	    return 0;
 	  case IVL_LPM_ADD:
 	  case IVL_LPM_CMP_EQ:
@@ -984,7 +1046,7 @@ extern "C" int ivl_lpm_signed(ivl_lpm_t net)
 	  case IVL_LPM_SHIFTL:
 	  case IVL_LPM_SHIFTR:
 	    return net->u_.shift.signed_flag;
-	    return 0;
+	  case IVL_LPM_DECODE:
 	  case IVL_LPM_UFUNC:
 	    return 0;
 	  default:
@@ -996,8 +1058,12 @@ extern "C" int ivl_lpm_signed(ivl_lpm_t net)
 extern "C" unsigned ivl_lpm_size(ivl_lpm_t net)
 {
       switch (net->type) {
+	  case IVL_LPM_DEMUX:
+	    return net->u_.demux.size;
 	  case IVL_LPM_MUX:
 	    return net->u_.mux.size;
+	  case IVL_LPM_RAM:
+	    return net->u_.ff.scnt;
 	  case IVL_LPM_UFUNC:
 	    return net->u_.ufunc.ports - 1;
 	  default:
@@ -1018,8 +1084,13 @@ extern "C" unsigned ivl_lpm_width(ivl_lpm_t net)
 	  case IVL_LPM_FF:
 	  case IVL_LPM_RAM:
 	    return net->u_.ff.width;
+          case IVL_LPM_LATCH:
+	    return net->u_.latch.width;
+	  case IVL_LPM_DECODE:
 	  case IVL_LPM_MUX:
 	    return net->u_.mux.width;
+	  case IVL_LPM_DEMUX:
+	    return net->u_.demux.width;
 	  case IVL_LPM_ADD:
 	  case IVL_LPM_CMP_EQ:
 	  case IVL_LPM_CMP_GE:
@@ -1046,7 +1117,7 @@ extern "C" ivl_memory_t ivl_lpm_memory(ivl_lpm_t net)
       assert(net);
       switch (net->type) {
 	  case IVL_LPM_RAM:
-	    return net->u_.ff.mem;
+	    return net->u_.ff.a.mem;
 	  default:
 	    assert(0);
 	    return 0;
@@ -1935,6 +2006,21 @@ extern "C" ivl_variable_type_t ivl_variable_type(ivl_variable_t net)
 
 /*
  * $Log: t-dll-api.cc,v $
+ * Revision 1.108.2.5  2006/04/16 19:26:40  steve
+ *  Fix handling of exploded memories with partial or missing resets.
+ *
+ * Revision 1.108.2.4  2006/03/26 23:09:24  steve
+ *  Handle asynchronous demux/bit replacements.
+ *
+ * Revision 1.108.2.3  2006/03/12 07:34:19  steve
+ *  Fix the memsynth1 case.
+ *
+ * Revision 1.108.2.2  2006/02/25 05:03:29  steve
+ *  Add support for negedge FFs by using attributes.
+ *
+ * Revision 1.108.2.1  2006/02/19 00:11:33  steve
+ *  Handle synthesis of FF vectors with l-value decoder.
+ *
  * Revision 1.108  2004/10/04 01:10:55  steve
  *  Clean up spurious trailing white space.
  *
