@@ -17,7 +17,7 @@
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 #ifdef HAVE_CVS_IDENT
-#ident "$Id: t-dll.cc,v 1.131 2004/10/04 01:10:55 steve Exp $"
+#ident "$Id: t-dll.cc,v 1.131.2.8 2006/07/23 19:42:35 steve Exp $"
 #endif
 
 # include "config.h"
@@ -33,6 +33,9 @@
 #endif
 # include  <stdlib.h>
 
+#include <new> // standard operator new
+using std::bad_alloc;
+
 #if defined(__WIN32__)
 
 inline ivl_dll_t ivl_dlopen(const char *name)
@@ -940,6 +943,8 @@ void dll_target::net_case_cmp(const NetCaseCmp*net)
       obj->delay[1] = net->fall_time();
       obj->delay[2] = net->decay_time();
 
+      logic_attributes(obj,net);
+
       scope_add_logic(scope, obj);
 }
 
@@ -965,6 +970,8 @@ bool dll_target::net_function(const NetUserFunc*net)
       obj->type = IVL_LPM_UFUNC;
       obj->name  = net->name();
       obj->scope = find_scope(des_, net->scope());
+      obj->attr = 0;
+      obj->nattr = 0;
       assert(obj->scope);
 
 	/* Get the definition of the function and save it. */
@@ -1104,9 +1111,11 @@ void dll_target::lpm_add_sub(const NetAddSub*net)
 	    obj->type = IVL_LPM_SUB;
       else
 	    obj->type = IVL_LPM_ADD;
-      obj->name = net->name(); // NetAddSub names are permallocated.
+      obj->name = net->name(); // NetAddSub names are permallocated
       assert(net->scope());
       obj->scope = find_scope(des_, net->scope());
+      obj->attr  = 0;
+      obj->nattr = 0;
       assert(obj->scope);
 
       obj->u_.arith.signed_flag = 0;
@@ -1176,6 +1185,8 @@ void dll_target::lpm_clshift(const NetCLShift*net)
       ivl_lpm_t obj = new struct ivl_lpm_s;
       obj->type = IVL_LPM_SHIFTL;
       obj->name = net->name();
+      obj->attr  = 0;
+      obj->nattr = 0;
       assert(net->scope());
       obj->scope = find_scope(des_, net->scope());
       assert(obj->scope);
@@ -1243,6 +1254,8 @@ void dll_target::lpm_clshift(const NetCLShift*net)
 void dll_target::lpm_compare(const NetCompare*net)
 {
       ivl_lpm_t obj = new struct ivl_lpm_s;
+      obj->attr  = 0;
+      obj->nattr = 0;
       obj->name = net->name(); // NetCompare names are permallocated
       assert(net->scope());
       obj->scope = find_scope(des_, net->scope());
@@ -1357,6 +1370,8 @@ void dll_target::lpm_divide(const NetDivide*net)
       ivl_lpm_t obj = new struct ivl_lpm_s;
       obj->type  = IVL_LPM_DIVIDE;
       obj->name  = net->name();
+      obj->attr  = 0;
+      obj->nattr = 0;
       assert(net->scope());
       obj->scope = find_scope(des_, net->scope());
       assert(obj->scope);
@@ -1427,6 +1442,8 @@ void dll_target::lpm_modulo(const NetModulo*net)
       ivl_lpm_t obj = new struct ivl_lpm_s;
       obj->type  = IVL_LPM_MOD;
       obj->name  = net->name();
+      obj->attr  = 0;
+      obj->nattr = 0;
       assert(net->scope());
       obj->scope = find_scope(des_, net->scope());
       assert(obj->scope);
@@ -1489,16 +1506,131 @@ void dll_target::lpm_modulo(const NetModulo*net)
       scope_add_lpm(obj->scope, obj);
 }
 
+bool dll_target::lpm_decode(const NetDecode*net)
+{
+      return true;
+}
+
+ivl_lpm_t dll_target::lpm_decode_ff_(const NetDecode*net)
+{
+      if (net == 0)
+	    return 0;
+
+      ivl_lpm_t obj = new struct ivl_lpm_s;
+      obj->type  = IVL_LPM_DECODE;
+      obj->name  = net->name();
+      obj->attr  = 0;
+      obj->nattr = 0;
+      obj->scope = find_scope(des_, net->scope());
+
+      obj->u_.mux.swid  = net->awidth();
+      obj->u_.mux.size  = 0;
+      obj->u_.mux.width = net->width();
+      obj->u_.mux.d = 0;
+
+      if (obj->u_.mux.swid > 1) {
+	    obj->u_.mux.s.pins = new ivl_nexus_t[obj->u_.mux.swid];
+
+	    for (unsigned idx = 0 ;  idx < obj->u_.mux.swid ; idx += 1) {
+		  const Nexus*nex = net->pin_Address(idx).nexus();
+		  assert(nex->t_cookie());
+		  obj->u_.mux.s.pins[idx] = (ivl_nexus_t) nex->t_cookie();
+		  nexus_lpm_add(obj->u_.mux.s.pins[idx], obj, idx,
+				IVL_DR_HiZ, IVL_DR_HiZ);
+	    }
+      } else {
+	    assert(obj->u_.mux.swid == 1);
+	    const Nexus*nex = net->pin_Address(0).nexus();
+	    assert(nex->t_cookie());
+	    obj->u_.mux.s.pin = (ivl_nexus_t) nex->t_cookie();
+	    nexus_lpm_add(obj->u_.mux.s.pin, obj, 0, IVL_DR_HiZ, IVL_DR_HiZ);
+      }
+
+      scope_add_lpm(obj->scope, obj);
+      return obj;
+}
+
+bool dll_target::lpm_demux(const NetDemux*net)
+{
+      unsigned idx;
+      unsigned width = net->width();
+      unsigned awid = net->awidth();
+      unsigned size = net->size();
+      ivl_lpm_t obj = new struct ivl_lpm_s;
+      obj->type  = IVL_LPM_DEMUX;
+      obj->name  = net->name();
+      obj->attr  = 0;
+      obj->nattr = 0;
+      obj->scope = find_scope(des_, net->scope());
+      assert(obj->scope);
+
+      obj->u_.demux.width = width;
+      obj->u_.demux.awid  = awid;
+      obj->u_.demux.size  = net->size();
+
+      ivl_nexus_t*tmp = new ivl_nexus_t [2*width + awid + width/size];
+      obj->u_.demux.q = tmp;
+      obj->u_.demux.d = tmp + width;
+      obj->u_.demux.a = tmp + 2*width;
+      obj->u_.demux.bit_in = tmp + 2*width + awid;
+
+      for (idx = 0 ;  idx < width ;  idx += 1) {
+	    const Nexus*nex = net->pin_Q(idx).nexus();
+	    obj->u_.demux.q[idx] = (ivl_nexus_t) nex->t_cookie();
+
+	      /* It is possible, but unlikely, that the Q is unused. */
+	    if (obj->u_.demux.q[idx])
+		  nexus_lpm_add(obj->u_.demux.q[idx], obj, idx,
+				IVL_DR_STRONG, IVL_DR_STRONG);
+      }
+
+      for (idx = 0 ;  idx < width ;  idx += 1) {
+	    const Nexus*nex = net->pin_Data(idx).nexus();
+	    assert(nex->t_cookie());
+	    obj->u_.demux.d[idx] = (ivl_nexus_t) nex->t_cookie();
+	    nexus_lpm_add(obj->u_.demux.d[idx], obj, idx,
+			  IVL_DR_HiZ, IVL_DR_HiZ);
+      }
+
+      for (idx = 0 ;  idx < awid ;  idx += 1) {
+	    const Nexus*nex = net->pin_Address(idx).nexus();
+	    assert(nex->t_cookie());
+	    obj->u_.demux.a[idx] = (ivl_nexus_t) nex->t_cookie();
+	    nexus_lpm_add(obj->u_.demux.a[idx], obj, idx,
+			  IVL_DR_HiZ, IVL_DR_HiZ);
+      }
+
+      for (idx = 0 ;  idx < width/size ;  idx += 1) {
+	    const Nexus*nex = net->pin_WriteData(idx).nexus();
+	    assert(nex->t_cookie());
+	    obj->u_.demux.bit_in[idx] = (ivl_nexus_t) nex->t_cookie();
+	    nexus_lpm_add(obj->u_.demux.bit_in[idx], obj, 0,
+			  IVL_DR_HiZ, IVL_DR_HiZ);
+      }
+
+      obj->nattr = net->attr_cnt();
+      obj->attr = fill_in_attributes(net);
+
+      scope_add_lpm(obj->scope, obj);
+      return true;
+}
+
 void dll_target::lpm_ff(const NetFF*net)
 {
       ivl_lpm_t obj = new struct ivl_lpm_s;
       obj->type  = IVL_LPM_FF;
       obj->name  = net->name();
+      obj->attr  = 0;
+      obj->nattr = 0;
       obj->scope = find_scope(des_, net->scope());
+      obj->u_.ff.a.decode = lpm_decode_ff_(net->get_demux());
       assert(obj->scope);
 
       obj->u_.ff.width = net->width();
 
+      obj->nattr = net->attr_cnt();
+      obj->attr = fill_in_attributes(net);
+
       scope_add_lpm(obj->scope, obj);
 
       const Nexus*nex;
@@ -1605,18 +1737,49 @@ void dll_target::lpm_ff(const NetFF*net)
       }
 }
 
+void dll_target::lpm_latch( const NetLatch *latchPtr )
+{
+  try
+    {
+      ivl_lpm_s *objPtr = new ivl_lpm_s;
+      objPtr->type = IVL_LPM_LATCH;
+      objPtr->name = latchPtr->name();
+      objPtr->scope = find_scope( des_, latchPtr->scope() );
+      assert( objPtr->scope ); // C++ programmers prefer using exceptions rather than assertions.
+
+      objPtr->u_.latch.width = latchPtr->width();
+
+      objPtr->nattr = latchPtr->attr_cnt();
+      objPtr->attr = fill_in_attributes( latchPtr );
+
+      scope_add_lpm( objPtr->scope, objPtr );
+    }
+  catch ( bad_alloc &memoryAllocationException )
+    {
+      cerr << "Exception occurred: " << memoryAllocationException.what() << endl;
+    }
+
+} // end function lpm_latch
+
 void dll_target::lpm_ram_dq(const NetRamDq*net)
 {
+      ivl_memory_t mem = find_memory(des_, net->mem());
+      assert(mem);
+
+      const NetNet*ereg = net->mem()->reg_from_explode();
+
       ivl_lpm_t obj = new struct ivl_lpm_s;
       obj->type  = IVL_LPM_RAM;
       obj->name  = net->name();
-      obj->u_.ff.mem = find_memory(des_, net->mem());
-      assert(obj->u_.ff.mem);
+      obj->attr  = 0;
+      obj->nattr = 0;
+      obj->u_.ff.a.mem = ereg? 0 : mem;
       obj->scope = find_scope(des_, net->mem()->scope());
       assert(obj->scope);
 
       obj->u_.ff.width = net->width();
       obj->u_.ff.swid = net->awidth();
+      obj->u_.ff.scnt = net->mem()->count();
 
       scope_add_lpm(obj->scope, obj);
 
@@ -1626,6 +1789,7 @@ void dll_target::lpm_ram_dq(const NetRamDq*net)
       // the clock input.
 
       bool has_write_port = net->pin_InClock().is_linked();
+      assert( ereg? !has_write_port : 1 );
 
       // Connect the write clock and write enable
 
@@ -1717,6 +1881,21 @@ void dll_target::lpm_ram_dq(const NetRamDq*net)
 				IVL_DR_STRONG, IVL_DR_STRONG);
 	    }
       }
+
+      if (ereg) {
+	    unsigned count = obj->u_.ff.width * obj->u_.ff.scnt;
+	    assert(ereg->pin_count() == count);
+
+	    obj->u_.ff.d.pins = new ivl_nexus_t [count];
+
+	    for (unsigned idx = 0 ;  idx < count ;  idx += 1) {
+		  nex = ereg->pin(idx).nexus();
+		  assert(nex->t_cookie());
+		  obj->u_.ff.d.pins[idx] = (ivl_nexus_t) nex->t_cookie();
+		  nexus_lpm_add(obj->u_.ff.d.pins[idx], obj, 0,
+				IVL_DR_HiZ, IVL_DR_HiZ);
+	    }
+      }
 }
 
 void dll_target::lpm_mult(const NetMult*net)
@@ -1724,6 +1903,8 @@ void dll_target::lpm_mult(const NetMult*net)
       ivl_lpm_t obj = new struct ivl_lpm_s;
       obj->type  = IVL_LPM_MULT;
       obj->name  = net->name();
+      obj->attr  = 0;
+      obj->nattr = 0;
       assert(net->scope());
       obj->scope = find_scope(des_, net->scope());
       assert(obj->scope);
@@ -1803,7 +1984,9 @@ void dll_target::lpm_mux(const NetMux*net)
 {
       ivl_lpm_t obj = new struct ivl_lpm_s;
       obj->type  = IVL_LPM_MUX;
-      obj->name  = net->name(); // The NetMux perallocates its name.
+      obj->name  = net->name(); // NetMux names are permallocated
+      obj->attr  = 0;
+      obj->nattr = 0;
       obj->scope = find_scope(des_, net->scope());
       assert(obj->scope);
 
@@ -1828,10 +2011,34 @@ void dll_target::lpm_mux(const NetMux*net)
 
 	    for (unsigned idx = 0 ;  idx < obj->u_.mux.width ;  idx += 1) {
 		  nex = net->pin_Result(idx).nexus();
-		  assert(nex->t_cookie());
+#if 0
+		  if (! nex->t_cookie()) {
+			cerr << net->get_line() << ": internal error: "
+			     << "broken mux. name=" << net->name()
+			     << ", width=" << obj->u_.mux.width
+			     << ", swid=" << obj->u_.mux.swid << endl;
+			for (unsigned tmp = 0; tmp < obj->u_.mux.width; tmp += 1) {
+			      const Nexus*tmpn = net->pin_Result(tmp).nexus();
+			      if (tmpn->t_cookie() == 0)
+				    continue;
+
+			      cerr << net->get_line() << ": XXXX "
+				   << "Result(" << tmp << ") : "
+				   << tmpn->name() << endl;
+			}
+		  }
+#endif
 		  obj->u_.mux.q.pins[idx] = (ivl_nexus_t) nex->t_cookie();
-		  nexus_lpm_add(obj->u_.mux.q.pins[idx], obj, 0,
-				IVL_DR_STRONG, IVL_DR_STRONG);
+
+		    /* It is possible (although unlikely) that the
+		       result bit of the mux is not used. This can
+		       happen, for example, if the output is
+		       overridden in the HDL. In that case, skip
+		       it. But if the bit is used, then add it to the
+		       nexus. */
+		  if (obj->u_.mux.q.pins[idx])
+			nexus_lpm_add(obj->u_.mux.q.pins[idx], obj, 0,
+				      IVL_DR_STRONG, IVL_DR_STRONG);
 	    }
       }
 
@@ -2180,6 +2387,30 @@ extern const struct target tgt_dll = { "dll", &dll_target_obj };
 
 /*
  * $Log: t-dll.cc,v $
+ * Revision 1.131.2.8  2006/07/23 19:42:35  steve
+ *  Handle statement output override better in blocks.
+ *
+ * Revision 1.131.2.7  2006/04/16 19:26:40  steve
+ *  Fix handling of exploded memories with partial or missing resets.
+ *
+ * Revision 1.131.2.6  2006/03/26 23:09:24  steve
+ *  Handle asynchronous demux/bit replacements.
+ *
+ * Revision 1.131.2.5  2006/03/12 07:34:19  steve
+ *  Fix the memsynth1 case.
+ *
+ * Revision 1.131.2.4  2006/02/25 05:03:29  steve
+ *  Add support for negedge FFs by using attributes.
+ *
+ * Revision 1.131.2.3  2006/02/19 00:11:34  steve
+ *  Handle synthesis of FF vectors with l-value decoder.
+ *
+ * Revision 1.131.2.2  2006/01/21 21:42:33  steve
+ *  When mux has wide select but sparse choices, use 1hot translation.
+ *
+ * Revision 1.131.2.1  2005/02/19 16:39:31  steve
+ *  Spellig fixes.
+ *
  * Revision 1.131  2004/10/04 01:10:55  steve
  *  Clean up spurious trailing white space.
  *
