@@ -19,7 +19,7 @@
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 #ifdef HAVE_CVS_IDENT
-#ident "$Id: t-dll.h,v 1.115 2004/10/04 01:10:56 steve Exp $"
+#ident "$Id: t-dll.h,v 1.115.2.5 2006/04/16 19:26:40 steve Exp $"
 #endif
 
 # include  "target.h"
@@ -76,8 +76,11 @@ struct dll_target  : public target_t, public expr_scan_t {
       void lpm_add_sub(const NetAddSub*);
       void lpm_clshift(const NetCLShift*);
       void lpm_compare(const NetCompare*);
+      bool lpm_decode(const NetDecode*);
+      bool lpm_demux(const NetDemux*);
       void lpm_divide(const NetDivide*);
       void lpm_ff(const NetFF*);
+      void lpm_latch( const NetLatch * );
       void lpm_modulo(const NetModulo*);
       void lpm_mult(const NetMult*);
       void lpm_mux(const NetMux*);
@@ -162,6 +165,8 @@ struct dll_target  : public target_t, public expr_scan_t {
 
       void add_root(ivl_design_s &des_, const NetScope *s);
 
+      ivl_lpm_t lpm_decode_ff_(const NetDecode*);
+
       void sub_off_from_expr_(long);
       void mul_expr_by_const_(long);
 
@@ -290,10 +295,14 @@ struct ivl_lpm_s {
       ivl_scope_t scope;
       perm_string name;
 
+      struct ivl_attribute_s*attr;
+      unsigned nattr;
+
       union {
 	    struct ivl_lpm_ff_s {
 		  unsigned width;
 		  unsigned swid; // ram only
+		  unsigned scnt;
 		  ivl_nexus_t clk;
 		  ivl_nexus_t we;
 		  ivl_nexus_t aclr;
@@ -312,11 +321,18 @@ struct ivl_lpm_s {
 			ivl_nexus_t*pins;
 			ivl_nexus_t pin;
 		  } s;
-		  ivl_memory_t mem; // ram only
+		  union {
+			ivl_memory_t mem; // ram only
+			ivl_lpm_t decode; // FF only
+		  } a;
 		  ivl_expr_t aset_value;
 		  ivl_expr_t sset_value;
 	    } ff;
 
+	    struct ivl_lpm_latch_s {
+	      unsigned width;
+	    } latch;
+
 	    struct ivl_lpm_mux_s {
 		  unsigned width;
 		  unsigned size;
@@ -332,6 +348,14 @@ struct ivl_lpm_s {
 		  } s;
 	    } mux;
 
+	    struct ivl_lpm_demux_s {
+		  unsigned width;
+		  unsigned awid;
+		  unsigned size;
+		  ivl_nexus_t*bit_in;
+		  ivl_nexus_t *q,*d,*a;
+	    } demux;
+
 	    struct ivl_lpm_shift_s {
 		  unsigned width;
 		  unsigned select;
@@ -684,6 +708,21 @@ struct ivl_variable_s {
 
 /*
  * $Log: t-dll.h,v $
+ * Revision 1.115.2.5  2006/04/16 19:26:40  steve
+ *  Fix handling of exploded memories with partial or missing resets.
+ *
+ * Revision 1.115.2.4  2006/03/26 23:09:25  steve
+ *  Handle asynchronous demux/bit replacements.
+ *
+ * Revision 1.115.2.3  2006/03/12 07:34:19  steve
+ *  Fix the memsynth1 case.
+ *
+ * Revision 1.115.2.2  2006/02/25 05:03:30  steve
+ *  Add support for negedge FFs by using attributes.
+ *
+ * Revision 1.115.2.1  2006/02/19 00:11:34  steve
+ *  Handle synthesis of FF vectors with l-value decoder.
+ *
  * Revision 1.115  2004/10/04 01:10:56  steve
  *  Clean up spurious trailing white space.
  *
