@@ -17,7 +17,7 @@
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 #ifdef HAVE_CVS_IDENT
-#ident "$Id: vvp_process.c,v 1.93 2004/10/04 01:10:57 steve Exp $"
+#ident "$Id: vvp_process.c,v 1.93.2.3 2007/02/26 19:51:40 steve Exp $"
 #endif
 
 # include  "vvp_priv.h"
@@ -161,18 +161,39 @@ static void assign_to_memory(ivl_memory_t mem, unsigned idx,
  * This function, in addition to setting the value into index 0, sets
  * bit 4 to 1 if the value is unknown.
  */
+void draw_eval_expr_into_integer(ivl_expr_t expr, unsigned ix)
+{
+      struct vector_info vec;
+      int word;
+
+      switch (ivl_expr_value(expr)) {
+
+	  case IVL_VT_VECTOR:
+	    vec = draw_eval_expr(expr, 0);
+	    fprintf(vvp_out, "    %%ix/get %u, %u, %u;\n",
+		    ix, vec.base, vec.wid);
+	    clr_vector(vec);
+	    break;
+
+	  case IVL_VT_REAL:
+	    word = draw_eval_real(expr);
+	    clr_word(word);
+	    fprintf(vvp_out, "    %%cvt/ir %u, %u;\n", ix, word);
+	    break;
+
+	  default:
+	    assert(0);
+      }
+}
+
 static void calculate_into_x0(ivl_expr_t expr)
 {
-      struct vector_info vec = draw_eval_expr(expr, 0);
-      fprintf(vvp_out, "    %%ix/get 0, %u, %u;\n", vec.base, vec.wid);
-      clr_vector(vec);
+      draw_eval_expr_into_integer(expr, 0);
 }
 
 static void calculate_into_x1(ivl_expr_t expr)
 {
-      struct vector_info vec = draw_eval_expr(expr, 0);
-      fprintf(vvp_out, "    %%ix/get 1, %u, %u;\n", vec.base, vec.wid);
-      clr_vector(vec);
+      draw_eval_expr_into_integer(expr, 1);
 }
 
 /*
@@ -429,7 +450,7 @@ static int show_stmt_assign(ivl_statement_t net)
  * This function handles the case of non-blocking assign to word
  * variables such as real, i.e:
  *
- *     read foo;
+ *     real foo;
  *     foo <= 1.0;
  *
  * In this case we know (by Verilog syntax) that there is only exactly
@@ -467,6 +488,8 @@ static int show_stmt_assign_nb_var(ivl_statement_t net)
       fprintf(vvp_out, "    %%assign/wr W_%s, %lu, %u;\n",
 	      vvp_word_label(var), delay, word);
 
+      clr_word(word);
+
       return 0;
 }
 
@@ -1379,7 +1402,7 @@ static int show_stmt_while(ivl_statement_t net, ivl_scope_t sscope)
       rc += show_statement(ivl_stmt_sub_stmt(net), sscope);
 
 	/* This is the bottom of the loop. branch to the top where the
-	   test is repeased, and also draw the out label. */
+	   test is repeated, and also draw the out label. */
       fprintf(vvp_out, "    %%jmp T_%d.%d;\n", thread_count, top_label);
       fprintf(vvp_out, "T_%d.%d ;\n", thread_count, out_label);
       clear_expression_lookaside();
@@ -1626,6 +1649,15 @@ int draw_func_definition(ivl_scope_t scope)
 
 /*
  * $Log: vvp_process.c,v $
+ * Revision 1.93.2.3  2007/02/26 19:51:40  steve
+ *  Spelling fixes (larry doolittle)
+ *
+ * Revision 1.93.2.2  2005/01/28 18:29:29  steve
+ *  Add ability to compile real values into index registers.
+ *
+ * Revision 1.93.2.1  2004/12/12 04:25:10  steve
+ *  Fix leak of word registers in code generator.
+ *
  * Revision 1.93  2004/10/04 01:10:57  steve
  *  Clean up spurious trailing white space.
  *
