@@ -17,7 +17,7 @@
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 #ifdef HAVE_CVS_IDENT
-#ident "$Id: vvp_scope.c,v 1.103 2004/10/04 01:10:57 steve Exp $"
+#ident "$Id: vvp_scope.c,v 1.103.2.7 2007/02/26 19:51:40 steve Exp $"
 #endif
 
 # include  "vvp_priv.h"
@@ -29,7 +29,7 @@
 # include  <string.h>
 
 /*
- *  Escape non-symbol chararacters in ids, and quotes in strings.
+ *  Escape non-symbol characters in ids, and quotes in strings.
  */
 
 inline static char hex_digit(unsigned i)
@@ -38,6 +38,18 @@ inline static char hex_digit(unsigned i)
       return i>=10 ? i-10+'A' : i+'0';
 }
 
+ivl_attribute_t find_lpm_attr(ivl_lpm_t net, const char*key)
+{
+      unsigned idx;
+      for (idx = 0 ;  idx < ivl_lpm_attr_cnt(net) ;  idx += 1) {
+	    ivl_attribute_t atr = ivl_lpm_attr_val(net, idx);
+	    if (strcmp(key,atr->key) == 0)
+		  return atr;
+      }
+
+      return 0;
+}
+
 const char *vvp_mangle_id(const char *id)
 {
       static char *out = 0x0;
@@ -383,8 +395,14 @@ static const char* draw_net_input_drive(ivl_nexus_t nex, ivl_nexus_ptr_t nptr)
       lpm = ivl_nexus_ptr_lpm(nptr);
       if (lpm) switch (ivl_lpm_type(lpm)) {
 
+	  case IVL_LPM_DECODE:
+	      /* The decoder has no outputs.
+		 (It is bound to other devices.) */
+	    break;
+
 	  case IVL_LPM_FF:
 	  case IVL_LPM_MUX:
+	  case IVL_LPM_DEMUX:
 	    for (idx = 0 ;  idx < ivl_lpm_width(lpm) ;  idx += 1)
 		  if (ivl_lpm_q(lpm, idx) == nex) {
 		     sprintf(result, "L_%s.%s/%u",
@@ -394,7 +412,29 @@ static const char* draw_net_input_drive(ivl_nexus_t nex, ivl_nexus_ptr_t nptr)
 		  }
 	    break;
 
+	      /* The output for a RAM depends on whether it is
+		 exploded or not. If it is, treat is like a MUX. If it
+		 is not, treat it like an arithmetic LPM. */
 	  case IVL_LPM_RAM:
+	    if (ivl_lpm_memory(lpm)) {
+		  for (idx = 0 ;  idx < ivl_lpm_width(lpm) ;  idx += 1) {
+			if (ivl_lpm_q(lpm, idx) == nex) {
+			      sprintf(result, "L_%p[%u]", lpm, idx);
+			      return result;
+			}
+		  }
+	    } else {
+		  for (idx = 0 ;  idx < ivl_lpm_width(lpm) ;  idx += 1) {
+			if (ivl_lpm_q(lpm, idx) == nex) {
+			      sprintf(result, "L_%s.%s/%u",
+				      vvp_mangle_id(ivl_scope_name(ivl_lpm_scope(lpm))),
+				      vvp_mangle_id(ivl_lpm_basename(lpm)), idx);
+			      return result;
+			}
+		  }
+	    }
+	    break;
+
 	  case IVL_LPM_ADD:
 	  case IVL_LPM_SHIFTL:
 	  case IVL_LPM_SHIFTR:
@@ -479,7 +519,7 @@ const char* draw_net_input(ivl_nexus_t nex)
 	    break;
 
 	      /* Catch the special cases that the nets are supply
-		 nets. Drive constant values uncomditionally. */
+		 nets. Drive constant values unconditionally. */
 	  case IVL_SIT_SUPPLY0:
 	    nex_private = "C<su0>";
 	    ivl_nexus_set_private(nex, nex_private);
@@ -1088,7 +1128,13 @@ static void draw_event_in_scope(ivl_event_t obj)
       }
 }
 
-inline static void draw_lpm_ram(ivl_lpm_t net)
+static void draw_lpm_ram_exploded(ivl_lpm_t net)
+{
+      fprintf(vvp_out, "; exploded ram port!\n");
+      draw_lpm_mux(net);
+}
+
+static void draw_lpm_ram(ivl_lpm_t net)
 {
       unsigned idx;
       unsigned width = ivl_lpm_width(net);
@@ -1097,6 +1143,11 @@ inline static void draw_lpm_ram(ivl_lpm_t net)
       ivl_nexus_t clk = ivl_lpm_clk(net);
       ivl_nexus_t pin;
 
+      if (mem == 0) {
+	    draw_lpm_ram_exploded(net);
+	    return;
+      }
+
       if (clk) {
 	    fprintf(vvp_out, "CLK_%p .event posedge, ", net);
 	    draw_input_from_net(clk);
@@ -1224,6 +1275,77 @@ static void draw_lpm_cmp(ivl_lpm_t net)
       fprintf(vvp_out, ";\n");
 }
 
+static void draw_lpm_decode(ivl_lpm_t net)
+{
+      unsigned idx;
+
+      fprintf(vvp_out, "L_%s.%s .decode/adr ",
+	      vvp_mangle_id(ivl_scope_name(ivl_lpm_scope(net))),
+	      vvp_mangle_id(ivl_lpm_basename(net)));
+
+      for (idx = 0 ;  idx < ivl_lpm_selects(net) ;  idx += 1) {
+	    ivl_nexus_t a = ivl_lpm_select(net, idx);
+	    if (idx > 0)
+		  fprintf(vvp_out, ", ");
+	    if (a) {
+		  draw_input_from_net(a);
+	    } else {
+		  fprintf(vvp_out, "C<0>");
+	    }
+      }
+
+      fprintf(vvp_out, ";\n");
+}
+
+/*
+ * Draw a demux as an address decoder and .decode bit slices.
+ */
+static void draw_lpm_demux(ivl_lpm_t net)
+{
+      unsigned idx, wbit;
+      unsigned width = ivl_lpm_width(net);
+      unsigned size = ivl_lpm_size(net);
+      unsigned word = width / size;
+
+      assert(width%size == 0);
+
+	/* Draw a .decode/adr node to do address decoding. */
+      draw_lpm_decode(net);
+
+      for (wbit = 0 ;  wbit < word ;  wbit += 1) {
+	    for (idx = wbit ;  idx < width ;  idx += word) {
+		  ivl_nexus_t nex;
+
+		    /* This is a demux bit slice idx */
+		  fprintf(vvp_out, "L_%s.%s/%u .demux ",
+			  vvp_mangle_id(ivl_scope_name(ivl_lpm_scope(net))),
+			  vvp_mangle_id(ivl_lpm_basename(net)), idx);
+
+		    /* Reference the address decoder... */
+		  fprintf(vvp_out, "L_%s.%s, %u, ",
+			  vvp_mangle_id(ivl_scope_name(ivl_lpm_scope(net))),
+			  vvp_mangle_id(ivl_lpm_basename(net)), idx/word);
+
+		    /* not-selected bit value. */
+		  nex = ivl_lpm_data(net, idx);
+		  if (nex)
+			draw_input_from_net(nex);
+		  else
+			fprintf(vvp_out, "C<z>");
+
+		    /* selected bit value. */
+		  fprintf(vvp_out, ", ");
+		  nex = ivl_lpm_datab(net, wbit);
+		  if (nex)
+			draw_input_from_net(nex);
+		  else
+			fprintf(vvp_out, "C<z>");
+
+		  fprintf(vvp_out, ";\n");
+	    }
+      }
+}
+
 /*
  * Draw == and != gates. This is done as XNOR functors to compare each
  * pair of bits. The result is combined with a wide and, or a NAND if
@@ -1274,38 +1396,145 @@ static void draw_lpm_eq(ivl_lpm_t net)
 static void draw_lpm_ff(ivl_lpm_t net)
 {
       ivl_expr_t aset_expr = 0;
+      ivl_expr_t sset_expr = 0;
       const char*aset_bits = 0;
+      const char*sset_bits = 0;
 
       unsigned width, idx;
+      ivl_attribute_t clock_pol = find_lpm_attr(net, "ivl:clock_polarity");
+      ivl_lpm_t decode = ivl_lpm_decode(net);
 
       width = ivl_lpm_width(net);
 
-	/*        Q   C   CE  D   RS  --> Q+ */
-      fprintf(vvp_out, "L_%s.%s/def .udp/sequ \"DFF\", 5, 2,"
-	      " \"?" "r" "1" "0" "00"    "0\","
-	      " \"?" "r" "1" "1" "00"    "1\","
-	      " \"?" "r" "1" "x" "00"    "x\","
-	      " \"0" "r" "x" "0" "00"    "0\","
-	      " \"1" "r" "x" "1" "00"    "1\","
-	      " \"?" "*" "0" "?" "00"    "-\","
-	      " \"?" "_" "?" "?" "00"    "-\","
-	      " \"?" "?" "?" "?" "01"    "1\","
-	      " \"?" "?" "?" "?" "1?"    "0\","
-	      " \"?" "?" "1" "?" "00"    "-\","
-	      " \"?" "?" "?" "?" "00"    "-\""
-	      ";\n",
-	      vvp_mangle_id(ivl_scope_name(ivl_lpm_scope(net))),
-	      vvp_mangle_id(ivl_lpm_basename(net)));
-
+      if (clock_pol) {
+	      /*        Q   C   CE  D   RS  --> Q+ */
+	    fprintf(vvp_out, "L_%s.%s/def .udp/sequ \"DFF\", 5, 2,"
+		    " \"?" "f" "1" "0" "00"    "0\","
+		    " \"?" "f" "1" "1" "00"    "1\","
+		    " \"?" "f" "1" "x" "00"    "x\","
+		    " \"0" "f" "x" "0" "00"    "0\","
+		    " \"1" "f" "x" "1" "00"    "1\","
+		    " \"?" "*" "0" "?" "00"    "-\","
+		    " \"?" "_" "?" "?" "00"    "-\","
+		    " \"?" "?" "?" "?" "01"    "1\","
+		    " \"?" "?" "?" "?" "1?"    "0\","
+		    " \"?" "?" "1" "?" "00"    "-\","
+		    " \"?" "?" "?" "?" "00"    "-\""
+		    ";\n",
+		    vvp_mangle_id(ivl_scope_name(ivl_lpm_scope(net))),
+		    vvp_mangle_id(ivl_lpm_basename(net)));
+      } else {
+	      /*        Q   C   CE  D   RS  --> Q+ */
+	    fprintf(vvp_out, "L_%s.%s/def .udp/sequ \"DFF\", 5, 2,"
+		    " \"?" "r" "1" "0" "00"    "0\","
+		    " \"?" "r" "1" "1" "00"    "1\","
+		    " \"?" "r" "1" "x" "00"    "x\","
+		    " \"0" "r" "x" "0" "00"    "0\","
+		    " \"1" "r" "x" "1" "00"    "1\","
+		    " \"?" "*" "0" "?" "00"    "-\","
+		    " \"?" "_" "?" "?" "00"    "-\","
+		    " \"?" "?" "?" "?" "01"    "1\","
+		    " \"?" "?" "?" "?" "1?"    "0\","
+		    " \"?" "?" "1" "?" "00"    "-\","
+		    " \"?" "?" "?" "?" "00"    "-\""
+		    ";\n",
+		    vvp_mangle_id(ivl_scope_name(ivl_lpm_scope(net))),
+		    vvp_mangle_id(ivl_lpm_basename(net)));
+      }
       aset_expr = ivl_lpm_aset_value(net);
       if (aset_expr) {
 	    assert(ivl_expr_width(aset_expr) == width);
 	    aset_bits = ivl_expr_bits(aset_expr);
       }
 
+      sset_expr = ivl_lpm_sset_value(net);
+      if (sset_expr) {
+	    assert(ivl_expr_width(sset_expr) == width);
+	    sset_bits = ivl_expr_bits(sset_expr);
+      }
+
+	/* If there is a synchronous clear or set, then draw a
+	   compound enable that includes the Sclr or Sset input. We
+	   will use this for the enable of the basic DFF. */
+
+      if (ivl_lpm_enable(net) && ivl_lpm_sync_clr(net)) {
+	    fprintf(vvp_out, "L_%s.%s/en .functor OR, ",
+		    vvp_mangle_id(ivl_scope_name(ivl_lpm_scope(net))),
+		    vvp_mangle_id(ivl_lpm_basename(net)));
+	    draw_input_from_net(ivl_lpm_enable(net));
+	    fprintf(vvp_out, ", ");
+	    draw_input_from_net(ivl_lpm_sync_clr(net));
+	    fprintf(vvp_out, ";\n");
+      }
+
+      if (ivl_lpm_enable(net) && ivl_lpm_sync_set(net)) {
+	    fprintf(vvp_out, "L_%s.%s/en .functor OR, ",
+		    vvp_mangle_id(ivl_scope_name(ivl_lpm_scope(net))),
+		    vvp_mangle_id(ivl_lpm_basename(net)));
+	    draw_input_from_net(ivl_lpm_enable(net));
+	    fprintf(vvp_out, ", ");
+	    draw_input_from_net(ivl_lpm_sync_set(net));
+	    fprintf(vvp_out, ";\n");
+      }
+
       for (idx = 0 ;  idx < width ;  idx += 1) {
 	    ivl_nexus_t tmp;
 
+	    if (ivl_lpm_sync_clr(net)) {
+		  fprintf(vvp_out, "L_%s.%s/din/%u .functor MUXZ, ",
+			  vvp_mangle_id(ivl_scope_name(ivl_lpm_scope(net))),
+			  vvp_mangle_id(ivl_lpm_basename(net)), idx);
+		  draw_input_from_net(ivl_lpm_data(net,idx));
+		  fprintf(vvp_out, ", C<0>, ");
+		  draw_input_from_net(ivl_lpm_sync_clr(net));
+		  fprintf(vvp_out, ", C<1>;\n");
+
+	    } else if (ivl_lpm_sync_set(net)) {
+		  fprintf(vvp_out, "L_%s.%s/din/%u .functor MUXZ, ",
+			  vvp_mangle_id(ivl_scope_name(ivl_lpm_scope(net))),
+			  vvp_mangle_id(ivl_lpm_basename(net)), idx);
+		  draw_input_from_net(ivl_lpm_data(net,idx));
+		  if (sset_bits[idx] == '1')
+			fprintf(vvp_out, ", C<1>, ");
+		  else
+			fprintf(vvp_out, ", C<0>, ");
+		  draw_input_from_net(ivl_lpm_sync_set(net));
+		  fprintf(vvp_out, ", C<1>;\n");
+	    }
+
+	      /* If there is a decoder for this FF, then create a
+		 decoder enable node. The .decode/en arguments are the
+		 label for the .decode/adr, a .decoder/en slice
+		 number (the address it responds to), the label for a
+		 decoder enable, and the label for a mass-write enable. */
+	    if (decode) {
+		  ivl_lpm_t dec = ivl_lpm_decode(net);
+		  fprintf(vvp_out, "L_%s.%s/dec/%u .decode/en L_%s.%s, %u",
+			  vvp_mangle_id(ivl_scope_name(ivl_lpm_scope(net))),
+			  vvp_mangle_id(ivl_lpm_basename(net)),
+			  idx,
+			  vvp_mangle_id(ivl_scope_name(ivl_lpm_scope(dec))),
+			  vvp_mangle_id(ivl_lpm_basename(dec)),
+			  idx / ivl_lpm_width(decode));
+
+		  tmp = ivl_lpm_enable(net);
+		  if (tmp) {
+			fprintf(vvp_out, ", ");
+			draw_input_from_net(tmp);
+		  } else {
+			fprintf(vvp_out, ", C<1> ");
+		  }
+		  fprintf(vvp_out, ", ");
+		  if (ivl_lpm_sync_clr(net)) {
+			draw_input_from_net(ivl_lpm_sync_clr(net));
+		  } else if (ivl_lpm_sync_set(net)) {
+			draw_input_from_net(ivl_lpm_sync_set(net));
+		  } else {
+			fprintf(vvp_out, "C<0>");
+		  }
+		  fprintf(vvp_out, ";\n");
+	    }
+
 	    fprintf(vvp_out, "L_%s.%s/%u .udp ",
 		    vvp_mangle_id(ivl_scope_name(ivl_lpm_scope(net))),
 		    vvp_mangle_id(ivl_lpm_basename(net)), idx);
@@ -1317,17 +1546,42 @@ static void draw_lpm_ff(ivl_lpm_t net)
 	    tmp = ivl_lpm_clk(net);
 	    draw_input_from_net(tmp);
 
+	      /* Draw the enable input. */
 	    tmp = ivl_lpm_enable(net);
 	    fprintf(vvp_out, ", ");
-	    if (tmp)
+	    if (ivl_lpm_decode(net)) {
+		    /* If there is a decoder, get the enable from a
+		       decoder node. */
+		  fprintf(vvp_out, "L_%s.%s/dec/%u",
+			  vvp_mangle_id(ivl_scope_name(ivl_lpm_scope(net))),
+			  vvp_mangle_id(ivl_lpm_basename(net)), idx);
+	    } else if (tmp && (ivl_lpm_sync_clr(net) || ivl_lpm_sync_set(net))) {
+		    /* If there is no decoder but an otherwise complex
+		       enable, get the enable from the calculated
+		       logic. */
+		  fprintf(vvp_out, "L_%s.%s/en",
+			  vvp_mangle_id(ivl_scope_name(ivl_lpm_scope(net))),
+			  vvp_mangle_id(ivl_lpm_basename(net)));
+	    } else if (tmp) {
+		    /* Draw a simple enable if that's what we got. */
 		  draw_input_from_net(tmp);
-	    else
+	    } else {
+		    /* Otherwise, permanently enable the DFF. */
 		  fprintf(vvp_out, "C<1>");
+	    }
 
-	    tmp = ivl_lpm_data(net, idx);
-	    assert(tmp);
-	    fprintf(vvp_out, ", ");
-	    draw_input_from_net(tmp);
+	      /* Draw the data input. MUX it with the sync input
+		 if there is one. */
+	    if (ivl_lpm_sync_clr(net) || ivl_lpm_sync_set(net)) {
+		  fprintf(vvp_out, ", L_%s.%s/din/%u",
+			  vvp_mangle_id(ivl_scope_name(ivl_lpm_scope(net))),
+			  vvp_mangle_id(ivl_lpm_basename(net)), idx);
+	    } else {
+		  tmp = ivl_lpm_data(net, idx);
+		  assert(tmp);
+		  fprintf(vvp_out, ", ");
+		  draw_input_from_net(tmp);
+	    }
 
 	      /* Connect reset input. This may be the Aclr input, or
 		 an Aset to zero. */
@@ -1501,6 +1755,14 @@ static void draw_lpm_in_scope(ivl_lpm_t net)
 	    draw_lpm_ufunc(net);
 	    return;
 
+	  case IVL_LPM_DECODE:
+	    draw_lpm_decode(net);
+	    return;
+
+	  case IVL_LPM_DEMUX:
+	    draw_lpm_demux(net);
+	    return;
+
 	  default:
 	    fprintf(stderr, "XXXX LPM not supported: %s.%s\n",
 		    ivl_scope_name(ivl_lpm_scope(net)), ivl_lpm_basename(net));
@@ -1624,6 +1886,27 @@ int draw_scope(ivl_scope_t net, ivl_scope_t parent)
 
 /*
  * $Log: vvp_scope.c,v $
+ * Revision 1.103.2.7  2007/02/26 19:51:40  steve
+ *  Spelling fixes (larry doolittle)
+ *
+ * Revision 1.103.2.6  2006/04/16 19:26:41  steve
+ *  Fix handling of exploded memories with partial or missing resets.
+ *
+ * Revision 1.103.2.5  2006/03/26 23:09:26  steve
+ *  Handle asynchronous demux/bit replacements.
+ *
+ * Revision 1.103.2.4  2006/03/12 07:34:20  steve
+ *  Fix the memsynth1 case.
+ *
+ * Revision 1.103.2.3  2006/02/25 05:03:30  steve
+ *  Add support for negedge FFs by using attributes.
+ *
+ * Revision 1.103.2.2  2006/02/19 00:11:35  steve
+ *  Handle synthesis of FF vectors with l-value decoder.
+ *
+ * Revision 1.103.2.1  2006/01/18 06:15:45  steve
+ *  Support DFF with synchronous inputs.
+ *
  * Revision 1.103  2004/10/04 01:10:57  steve
  *  Clean up spurious trailing white space.
  *
