@@ -17,7 +17,7 @@
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 #ifdef HAVE_CVS_IDENT
-#ident "$Id: verinum.cc,v 1.43 2004/05/18 18:43:15 steve Exp $"
+#ident "$Id: verinum.cc,v 1.43.2.4 2007/03/23 23:02:31 steve Exp $"
 #endif
 
 # include "config.h"
@@ -41,7 +41,7 @@ verinum::verinum(const V*bits, unsigned nbits, bool has_len)
       }
 }
 
-verinum::verinum(const string&str)
+verinum::verinum(const std::string&str)
 : has_len_(true), has_sign_(false), string_flag_(true)
 {
       nbits_ = str.length() * 8;
@@ -257,7 +257,7 @@ signed long verinum::as_long() const
       return val;
 }
 
-string verinum::as_string() const
+std::string verinum::as_string() const
 {
       assert( nbits_%8 == 0 );
       if (nbits_ == 0)
@@ -283,7 +283,7 @@ string verinum::as_string() const
       }
 
       tmp[nbits_/8] = 0;
-      string result = string(tmp);
+      std::string result = std::string(tmp);
       delete[]tmp;
       return result;
 }
@@ -317,6 +317,20 @@ bool verinum::is_zero() const
       return true;
 }
 
+bool verinum::is_negative() const
+{
+      if (! has_sign_)
+	    return false;
+
+      if (nbits_ == 0)
+	    return false;
+
+      if (bits_[nbits_-1] == verinum::V1)
+	    return true;
+
+      return false;
+}
+
 /*
  * This function returns a version of the verinum that has only as
  * many bits as are needed to accurately represent the value. It takes
@@ -370,7 +384,7 @@ verinum trim_vnum(const verinum&that)
       return tmp;
 }
 
-ostream& operator<< (ostream&o, verinum::V v)
+std::ostream& operator<< (std::ostream&o, verinum::V v)
 {
       switch (v) {
 	  case verinum::V0:
@@ -393,7 +407,7 @@ ostream& operator<< (ostream&o, verinum::V v)
  * This operator is used by various dumpers to write the verilog
  * number in a Verilog format.
  */
-ostream& operator<< (ostream&o, const verinum&v)
+std::ostream& operator<< (std::ostream&o, const verinum&v)
 {
       if (v.is_string()) {
 	    o << "\"" << v.as_string() << "\"";
@@ -474,6 +488,18 @@ verinum::V operator == (const verinum&left, const verinum&right)
 
 verinum::V operator <= (const verinum&left, const verinum&right)
 {
+      if (left.has_sign() && right.has_sign()) {
+	    if (!left.is_defined())
+		  return verinum::Vx;
+	    if (!right.is_defined())
+		  return verinum::Vx;
+	    long diff = left.as_long() - right.as_long();
+	    if (diff <= 0)
+		  return verinum::V1;
+	    else
+		  return verinum::V0;
+      }
+
       unsigned idx;
       for (idx = left.len() ; idx > right.len() ;  idx -= 1) {
 	    if (left[idx-1] != verinum::V0) return verinum::V0;
@@ -501,6 +527,18 @@ verinum::V operator <= (const verinum&left, const verinum&right)
 
 verinum::V operator < (const verinum&left, const verinum&right)
 {
+      if (left.has_sign() && right.has_sign()) {
+	    if (!left.is_defined())
+		  return verinum::Vx;
+	    if (!right.is_defined())
+		  return verinum::Vx;
+	    long diff = left.as_long() - right.as_long();
+	    if (diff < 0)
+		  return verinum::V1;
+	    else
+		  return verinum::V0;
+      }
+
       unsigned idx;
       for (idx = left.len() ; idx > right.len() ;  idx -= 1) {
 	    if (left[idx-1] != verinum::V0) return verinum::V0;
@@ -902,6 +940,24 @@ verinum operator % (const verinum&left, const verinum&right)
       return result;
 }
 
+verinum concat(const verinum&left, const verinum&right)
+{
+      if (left.is_string() && right.is_string()) {
+	    std::string tmp = left.as_string() + right.as_string();
+	    verinum res (tmp);
+	    return res;
+      }
+
+      verinum res (verinum::V0, left.len() + right.len());
+      for (unsigned idx = 0 ;  idx < right.len() ;  idx += 1)
+	    res.set(idx, right.get(idx));
+
+      for (unsigned idx = 0 ;  idx < left.len() ;  idx += 1)
+	    res.set(idx+right.len(), left.get(idx));
+
+      return res;
+}
+
 verinum::V operator | (verinum::V l, verinum::V r)
 {
       if (l == verinum::V1)
@@ -942,6 +998,18 @@ verinum::V operator ^ (verinum::V l, verinum::V r)
 
 /*
  * $Log: verinum.cc,v $
+ * Revision 1.43.2.4  2007/03/23 23:02:31  steve
+ *  Fix compile time eval of <= comparison.
+ *
+ * Revision 1.43.2.3  2007/03/23 20:59:26  steve
+ *  Fix compile time evaluation of < operator.
+ *
+ * Revision 1.43.2.2  2005/12/07 03:28:44  steve
+ *  Support constant concatenation of constants.
+ *
+ * Revision 1.43.2.1  2005/08/13 00:45:55  steve
+ *  Fix compilation warnings/errors with newer compilers.
+ *
  * Revision 1.43  2004/05/18 18:43:15  steve
  *  Handle null string as a single nul character.
  *
