@@ -19,7 +19,7 @@
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 #ifdef HAVE_CVS_IDENT
-#ident "$Id: verinum.h,v 1.26 2004/02/17 06:52:55 steve Exp $"
+#ident "$Id: verinum.h,v 1.26.2.4 2007/03/23 20:59:26 steve Exp $"
 #endif
 
 # include  <string>
@@ -31,6 +31,8 @@
 class ostream;
 #endif
 
+using namespace std;
+
 /*
  * Numbers in Verilog are multibit strings, where each bit has 4
  * possible values: 0, 1, x or z. The verinum number is store in
@@ -43,7 +45,7 @@ class verinum {
       enum V { V0 = 0, V1, Vx, Vz };
 
       verinum();
-      verinum(const string&str);
+      verinum(const std::string&);
       verinum(const V*v, unsigned nbits, bool has_len =true);
       verinum(V, unsigned nbits =1, bool has_len =true);
       verinum(unsigned long val, unsigned bits);
@@ -72,6 +74,7 @@ class verinum {
 	// A number is "defined" if there are no x or z bits in its value.
       bool is_defined() const;
       bool is_zero() const;
+      bool is_negative() const;
 
 	// A number is "a string" if its value came directly from
 	// an ASCII description instead of a number value.
@@ -90,7 +93,7 @@ class verinum {
 
       unsigned long as_ulong() const;
       signed long   as_long() const;
-      string as_string() const;
+      std::string as_string() const;
 
     private:
       V* bits_;
@@ -107,8 +110,8 @@ class verinum {
    needed to accurately represent the contained value, signed or not. */
 extern verinum trim_vnum(const verinum&);
 
-extern ostream& operator<< (ostream&, const verinum&);
-extern ostream& operator<< (ostream&, verinum::V);
+extern std::ostream& operator<< (std::ostream&, const verinum&);
+extern std::ostream& operator<< (std::ostream&, verinum::V);
 
 extern verinum::V operator | (verinum::V l, verinum::V r);
 extern verinum::V operator & (verinum::V l, verinum::V r);
@@ -142,11 +145,25 @@ extern verinum operator % (const verinum&left, const verinum&right);
 extern verinum operator<< (const verinum&left, unsigned shift);
 extern verinum operator>> (const verinum&left, unsigned shift);
 
+extern verinum concat(const verinum&left, const verinum&right);
+
 /* Bitwise not returns the ones complement. */
 extern verinum v_not(const verinum&left);
 
 /*
  * $Log: verinum.h,v $
+ * Revision 1.26.2.4  2007/03/23 20:59:26  steve
+ *  Fix compile time evaluation of < operator.
+ *
+ * Revision 1.26.2.3  2005/12/07 03:28:45  steve
+ *  Support constant concatenation of constants.
+ *
+ * Revision 1.26.2.2  2005/08/13 00:45:55  steve
+ *  Fix compilation warnings/errors with newer compilers.
+ *
+ * Revision 1.26.2.1  2005/06/14 15:33:54  steve
+ *  Fix gcc4 build issues.
+ *
  * Revision 1.26  2004/02/17 06:52:55  steve
  *  Support unsigned divide of huge numbers.
  *
